/*
 * Arena.java
 *
 * Created on 25. marts 2004, 12:52
 */

package Game;

import java.awt.*;
import java.util.ArrayList;
import java.security.*;
import java.net.*;
import java.io.*;
import java.nio.*;
import java.nio.channels.*;
import java.nio.charset.*;


/** Describes an arena */
public final class Arena {
    
    
    abstract class FakeObject {
        DefaultObject real = null;
        private Color color = new Color(190, 190, 190); // Light gray
        private int x = 30, y = 30;
        private int angle = 0;
        
        /** Only the Arena class may create fake objects */
        private FakeObject() {
        }
        
        abstract void setReal();
        
        boolean isBot() {
            return false;
        }
        
        boolean isBotMarker() {
            if (isBot() && (real == null || !real.isCustom()))
                return true;
            else
                return false;
        }
        
        boolean isObstacle() {
            return false;
        }
        
        int getX() {
            return x;
        }
        
        void setX(int x) {
            this.x = x;
            real.x = x;
            checkCollision();
        }
        
        int getY() {
            return y;
        }
        
        void setY(int y) {
            this.y = y;
            real.y = y;
            checkCollision();
        }
        
        int getAngle() {
            return angle;
        }
        
        void setAngle(int angle) {
            this.angle = angle;
            real.angle = angle;
            ((Bot) real).turretAngle = angle;
        }
        
        Color getColor() {
            return color;
        }
        
        void setColor(Color color) {
            color = color;
            real.color = color;
        }
        
        void remove(){
            objects.remove(this);
            Game.kill(real);
        }
    }
    
    
    class FakeBot extends FakeObject {
        private boolean hidden = false;
        
        private FakeBot(int x, int y, int angle) {
            real = new Bot();
            setColor(new Color(190, 190, 190)); // Light gray
            setX(x);
            setY(y);
            setAngle(angle);
        }
        
        boolean isBot() {
            return true;
        }
        
        void hide() {
            if (!real.isCustom()) {
                Game.kill(real);
                hidden = true;
            } else {
                Setup.notice("BUG! In Arena.hide: Not a marker:" + real.toString());
            }
        }
        
        void unhide() {
            if (hidden)
                Game.spawn(real);
        }
        
        void toRealBot(Bot bot) {
            if (real != null)
                Game.kill(real);
            real = bot;
            setColor(getColor());
            setX(getX());
            setY(getY());
            setAngle(getAngle());
            Game.spawn(real);
        }
        
        void toBotMarker() {
            if (real != null) {
                Game.kill(real);
                if (!real.isCustom())
                    Setup.notice("BUG! In Arena.toBotMarker(): is already a marker!");
            } else
                Setup.notice("BUG! In Arena.toBotMarker(): real is null");
            real = new Bot();
            setColor(new Color(190, 190, 190)); // Light gray
            setX(getX());
            setY(getY());
            setAngle(getAngle());
            Game.spawn(real);
        }
        
        public String toString() {
            if (real != null)
                return real.toString();
            else {
                Setup.notice("Bug! In Arena.toString: Real is null");
                return "Bug!";
            }
        }
        
        
        void setReal() {
            real = new Bot();
            setColor(new Color(190, 190, 190)); // Light gray
            setX(getX());
            setY(getY());
            setAngle(getAngle());
        }
        
    }
    
    
    class FakeObstacle extends FakeObject {
        private int width = 5, height = 5;
        
        private FakeObstacle() {
            real = new Obstacle();
        }
        
        private FakeObstacle(int x, int y, int width, int height) {
            real = new Obstacle();
            setX(x);
            setY(y);
            setWidth(width);
            setHeight(height);
        }
        
        boolean isObstacle() {
            return true;
        }
        
        int getWidth() {
            return width;
        }
        
        void setWidth(int width) {
            this.width = width;
            real.width = width;
            checkCollision();
        }
        
        int getHeight() {
            return height;
        }
        
        void setHeight(int height) {
            this.height = height;
            real.height = height;
            checkCollision();
        }
        
        public String toString() {
            return "Obstacle";
        }
        
        void setReal() {
            real = new Obstacle();
            setX(getX());
            setY(getY());
            setWidth(getWidth());
            setHeight(getHeight());
        }
        
    }
    
    
    ArrayList botsStuck = new ArrayList();
    private String name = null;
    private String changedName = null;
    ArrayList objects = new ArrayList();
    private FakeObstacle leftWall, rightWall, topWall, buttomWall;
    private int width = 500, height = 500; // Default size
    
    
    /** Creates a new instance of Arena */
    Arena() {
        spawnAll();
    }
    
    
    void updateWalls() {
        
        if (leftWall == null)
            leftWall = new FakeObstacle();
        leftWall.setX(5);
        leftWall.setY(height / 2);
        leftWall.setWidth(10);
        leftWall.setHeight(height);
        
        if (rightWall == null)
            rightWall = new FakeObstacle();
        rightWall.setX(width - 5);
        rightWall.setY(height / 2);
        rightWall.setWidth(10);
        rightWall.setHeight(height);
        
        if (topWall == null)
            topWall = new FakeObstacle();
        topWall.setX(width / 2);
        topWall.setY(5);
        topWall.setWidth(width);
        topWall.setHeight(10);
        
        if (buttomWall == null)
            buttomWall = new FakeObstacle();
        buttomWall.setX(width / 2);
        buttomWall.setY(height - 5);
        buttomWall.setWidth(width);
        buttomWall.setHeight(10);
    }
    
    
    void spawnAll() {
        updateWalls();
        Game.spawn(leftWall.real);
        Game.spawn(rightWall.real);
        Game.spawn(topWall.real);
        Game.spawn(buttomWall.real);
        
        for (int i = 0; i < objects.size(); i++) {
            FakeObject object = ((FakeObject) objects.get(i));
            
            if (object.real != null) {
                Game.kill(object.real);
                if (object.real.isCustom())
                    Game.nrOfBotsLeft++;
            } else
                Setup.notice("BUG! In Arena.spawnAll: object.real is null");
            //object.setReal();
            
            if (object.isBot()) {
                FakeBot fakebot = (FakeBot) object;
                Bot realbot = (Bot) ((FakeBot) object).real;
                
                // Reset bot
                fakebot.setX(fakebot.getX());
                fakebot.setY(fakebot.getY());
                fakebot.setAngle(fakebot.getAngle());
                realbot.health = realbot.healthStart;
                realbot.isDead = false;
                realbot.isGhost = false;
                realbot.width = 30;
                realbot.height = 30;
                realbot.moveQueue.clear();
                realbot.rotateQueue.clear();
                realbot.lookQueue.clear();
                realbot.actionQueue.clear();
                realbot.sayQueue.clear();
                
            }
            
            Game.spawn(object.real);
        }
    }
    
    
    void removeAll() {
        for (int i = 0; i < objects.size(); i++) {
            FakeObject object = ((FakeObject) objects.get(i));
            Game.kill(object.real);
            object.setReal();
        }
    }
    
    
    /** Check all bots in this arena for collision, and if so mark them red */
    void checkCollision() {
        for (int i = 0; i < objects.size(); i++) {
            FakeObject object = (FakeObject) objects.get(i);
            if (object.isBot()) {
                
                // Check if the it has a graphical representation, and if it doesn't then set it. Only used to get the "real" with and height though
                if (object.real == null)
                    object.setReal();
                
                // Check if bot is outside arena
                if (object.x > width - object.real.getWidth() + 5) {
                    object.setX((int) (width - object.real.getWidth()) + 5);
                }
                if (object.y > height - object.real.getHeight() + 5) {
                    object.setY((int) (height - object.real.getHeight()  + 5));
                }
                
                // Check if the bot is, or was, colliding with something
                if (object.real.isColliding()) {
                    if (!botsStuck.contains(object)) {
                        object.real.color = Color.red;
                        botsStuck.add(object);
                    }
                } else {
                    if (botsStuck.contains(object)) {
                        object.setColor(object.getColor());
                        botsStuck.remove(object);
                    }
                }
            }
        }
    }
    
    
    /** Create a new obstacle marker */
    FakeObstacle makeObstacle(int x, int y, int width, int height) {
        FakeObstacle obstacle = new FakeObstacle(x, y, width, height);
        objects.add(obstacle);
        Game.spawn(obstacle.real);
        return obstacle;
    }
    
    
    /** Create a new bot marker */
    FakeBot makeBot(int x, int y, int angle) {
        FakeBot bot = new FakeBot(x, y, angle);
        objects.add(bot);
        Game.spawn(bot.real);
        return bot;
    }
    
    
    /** The name of this arena */
    public String toString() {
        return getName();
    }
    
    
    /** Save this arena to the disk in the bots directory. Override if it already exists. If name has changed, delete the old one */
    void save() {
        // Remove the real object references to minimize the file.
        for (int i = 0; i < objects.size(); i++) {
            ((FakeObject) objects.get(i)).real = null;
        }
        
        // Find the Arenas directory
        Class cls = this.getClass();
        ProtectionDomain pDomain = cls.getProtectionDomain();
        CodeSource cSource = pDomain.getCodeSource();
        URL thisDir = cSource.getLocation();
        
        File arenaFile = new File(thisDir.getPath() + File.separator + "Arenas" + File.separator + name + ".arena");
        
        // This arena has been renamed, delete the old file and clear the renamed flag
        if (changedName != null) {
            if (name != null && !arenaFile.delete()) Setup.notice("Could not remove the file: " + arenaFile.getAbsolutePath()); // Delete old file
            arenaFile = new File(thisDir.getPath() + File.separator + "Arenas" + File.separator + changedName + ".arena");
            name = changedName;
            changedName = null;
        }
        
        // Save the file
        String arenaText = "ARENA, " + width + ", " + height;
        
        for (int i = 0; i < objects.size(); i++) {
            FakeObject fo = (FakeObject) objects.get(i);
            if (fo.isBot()) {
                arenaText = arenaText.concat("\nBOT, " + fo.x + ", " + fo.y + ", " + fo.angle);
            } else {
                arenaText = arenaText.concat("\nOBSTACLE, " + fo.x + ", " + fo.y + ", " + fo.angle + ", " + ((FakeObstacle) fo).width + ", " + ((FakeObstacle) fo).height);
            }
        }
        
        try {
            FileChannel fileChannel = new RandomAccessFile(arenaFile, "rw").getChannel();
            fileChannel.write(ByteBuffer.wrap(arenaText.getBytes()));
            fileChannel.close();
        } catch (FileNotFoundException e) {
            Setup.error(e);
        } catch (IOException e) {
            Setup.error(e);
        }
        
    }
    
    
    boolean load(File file) {
        String allText = "";
        try {
            FileChannel fileChannel = new RandomAccessFile(file, "rw").getChannel();
            ByteBuffer bb = fileChannel.map(FileChannel.MapMode.READ_ONLY, 0, fileChannel.size());
            String encoding = System.getProperty("file.encoding");
            //String encoding = "UTF-8";
            CharBuffer cb = Charset.forName(encoding).decode(bb);
            allText = cb.toString();
            fileChannel.close();
        } catch (FileNotFoundException e) {
            Setup.error(e);
        } catch (IOException e) {
            Setup.error(e);
        }
        
        if (allText.length() == 0 || !allText.startsWith("ARENA")) {
            return false;
        } else {
            
            name = file.getName().substring(0, file.getName().length() - 6);
            
            String[] allInformation = allText.split("\n");
            for (int i = 0; i < allInformation.length; i++) {
                String[] info = allInformation[i].split(", ");
                try {
                    
                    if (info[0].startsWith("ARENA")) {
                        width = Integer.parseInt(info[1]);
                        height = Integer.parseInt(info[2]);
                        
                    } else if (info[0].startsWith("BOT")) {
                        makeBot(Integer.parseInt(info[1]), Integer.parseInt(info[2]), Integer.parseInt(info[3])); // x, y, angle
                        
                    } else if (info[0].startsWith("OBSTACLE")) {
                        makeObstacle(Integer.parseInt(info[1]), Integer.parseInt(info[2]), Integer.parseInt(info[4]), Integer.parseInt(info[5])); // x, y, (angle), width, height
                    } else {
                        throw new Exception();
                    }
                    
                } catch (Exception e) {
                    Setup.error("The file: " + file.getAbsolutePath() + " seems to be corrupt.\nIf it is not an arena file don't be a wanker and call it .arena!", e);
                    removeAll();
                    return false;
                }
                
            }
            
            spawnAll();
            return true;
        }
    }
    
    /** Delete this arena from the disk */
    void delete() {
        if (name != null) {
            // Find the Arenas directory
            Class cls = this.getClass();
            ProtectionDomain pDomain = cls.getProtectionDomain();
            CodeSource cSource = pDomain.getCodeSource();
            URL thisDir = cSource.getLocation();
            
            File arenaFile = new File(thisDir.getPath() + File.separator + "Arenas" + File.separator + name + ".arena");
            if (!arenaFile.delete()) Setup.notice("Could not remove the file: " + arenaFile.getAbsolutePath());
        }
    }
    
    
    public boolean equals(Object object) {
        return ((Arena) object).getName().equals(getName());
    }
    
    
    String getName() {
        return changedName == null ? name : changedName;
    }
    
    
    void setName(String newName) {
        changedName = newName;
    }
    
    
    int getWidth() {
        return width;
    }
    
    
    void setWidth(int width) {
        this.width = width;
        updateWalls();
    }
    
    
    
    int getHeight() {
        return height;
    }
    
    
    void setHeight(int height) {
        this.height = height;
        updateWalls();
    }
    
}