/*
 * Pacman.java
 *
 * Created on 19. februar 2004, 18:36
 */


package Game;

import java.awt.Graphics2D;
import java.awt.geom.Arc2D;
import java.awt.geom.RoundRectangle2D;
import java.awt.geom.AffineTransform;
import java.awt.Color;
import java.awt.Font;
import java.util.ArrayList;
import java.util.TreeSet;


/** A killer pacman */
public class Bot extends DefaultObject {
    
    
    // Actions
    private static final int STOP = 0;     // Do nothing
    private static final int FORWARD = 1;  // move relative to angle
    private static final int BACKWARD = 2; // move relative to angle
    private static final int RIGHT = 3;    // strafe or turn relative to angle
    private static final int LEFT = 4;     // strafe or turn relative to angle
    private static final int FIRE = 5;     // BANG!
    
    private static final int moveSpeed = 4;     // Heigher means faster movement
    private static final int turnSpeed = 5;     // Heigher means quicker turns
    
    double turretAngle = 0;
    
    // Orders
    private int move = STOP;   // STOP, FORWARD, BACKWARD, RIGHT, LEFT
    private int rotate = STOP;   // STOP, RIGHT, LEFT
    private int look = STOP;   // STOP, RIGHT, LEFT
    private int action = STOP; //STOP, FIRE
    
    // Queue orders
    ArrayList moveQueue = new ArrayList();
    ArrayList rotateQueue = new ArrayList();
    ArrayList lookQueue = new ArrayList();
    ArrayList actionQueue = new ArrayList();
    ArrayList sayQueue = new ArrayList();
    
    // Used to calculate lenth of orders
    private double moveDistance = 0;
    private double angleAmount = 0;
    private double turretAngleAmount = 0;
    private int fireNumber = 0;
    
    private static final int sayTimerStart = 65;   // How long the tekst will be shown on screen. Dissapears rather quicly on death even if high though
    private int sayTimer = 0;                      // Is set to saySetTimer and ticks down
    
    private static final int CLOSE = 0;            // Indicates that animated packman's mouth is closed.
    private static final int OPEN = 1;             // Indicates that animated packman's mouth is open.
    private int angleStart = 45;                   // The starting angle of the packman
    private int angleExtent = 270;                 // The extent of the angle of the packman
    private int mouth = CLOSE;                     // Current state of the mouth
    
    private Pistol pistol = new Bot.Pistol();      // The definition of the weapon type Gun
    private BombThrower bombs = new BombThrower(); // The The definition of the weapon type Bombs
    private Minigun minigun = new Minigun();
    private Shutgun shutgun = new Shutgun();
    private RocketLauncher rockets = new Bot.RocketLauncher();
    private Sniper sniper = new Sniper();
    private Weapon weapon = pistol;                // The weapon currently used by the pacman
    private int cooldown = 0;                      // Current cooldown. Ticks down if > 0
    
    private DefaultObject objectInSight = this;    // Prevent nasty null exceptions. May produce funky results though!
    
    private int tick = 0;
    
    private boolean moveBackOrForth = true;
    
    private class MoveElement {
        boolean moveBackOrForth = true;
        double distance = 0;
        
        MoveElement(double dist, boolean isForward) {
            distance = dist;
            moveBackOrForth = isForward;
        }
    }
    
    /** The funky arraylist thingy */
    public final class IdList {
        
        private final class IdListObject {
            double key = 0;
            boolean boolvalue = false;
            double realvalue = 0;
            String stringvalue = "";
            boolean isReal = false;
            boolean isBool = false;
            
            IdListObject(double key) {
                this.key = key;
            }
            
            IdListObject(double key, boolean value) {
                boolvalue = value;
                isReal = false;
                isBool = true;
                this.key = key;
            }
            
            IdListObject(double key, double value) {
                realvalue = value;
                isReal = true;
                isBool = false;
                this.key = key;
            }
            
            IdListObject(double key, String value) {
                stringvalue = value;
                isReal = false;
                isBool = false;
                this.key = key;
            }
            
            public boolean equals(Object object) {
                if (key == ((IdListObject) object).key)
                    return true;
                return false;
            }
        }
        
        
        private final ArrayList idList = new ArrayList();
        
        public void add(double id, double real) {
            idList.add(new IdListObject(id, real));
        }
        
        public void add(double id, boolean bool) {
            idList.add(new IdListObject(id, bool));
        }
        
        public void add(double id, String string) {
            idList.add(new IdListObject(id, string));
        }
        
        public double getReal(double id) {
            return ((IdListObject) idList.get(idList.indexOf(new IdListObject(id)))).realvalue;
        }
        
        public boolean getBool(double id) {
            return ((IdListObject) idList.get(idList.indexOf(new IdListObject(id)))).boolvalue;
        }
        
        public String getString(double id) {
            return ((IdListObject) idList.get(idList.indexOf(new IdListObject(id)))).stringvalue;
        }
        
        public boolean isReal(double id) {
            return ((IdListObject) idList.get(idList.indexOf(new IdListObject(id)))).isReal;
        }
        
        public boolean isBool(double id) {
            return ((IdListObject) idList.get(idList.indexOf(new IdListObject(id)))).isBool;
        }
    }
    
    
    /** Abstract inner class which describes a weapon */
    private abstract class Weapon {
        abstract void fire();
    }
    
    
    /** A pistol which fires standard bullets */
    private final class Pistol extends Weapon {
        private static final int cooldownTime = 30; // Cooldown of gun. Able to fire when 0. A possitive number means having to wait
        
        void fire() {
            Bullet bullet = new Bullet(x, y, turretAngle, Color.RED, Bot.this);
            bullet.firePower = 25;
            spawn(bullet);
            cooldown = cooldownTime;
        }
    }
    
    
    /** A minigun which shoots very fast (but less fatal) bullets  */
    private final class Minigun extends Weapon {
        private static final int cooldownTime = 5; // Cooldown of gun. Able to fire when 0. A possitive number means having to wait
        
        void fire() {
            Bullet bullet = new Bullet(x, y, turretAngle, Color.RED, Bot.this);
            bullet.firePower = 2;
            spawn(bullet);
            cooldown = cooldownTime;
        }
    }
    
    
    /** Will place a bomb */
    private final class BombThrower extends Weapon {
        private static final int cooldownTime = 50; // Cooldown of BombThrower. Able to fire when 0. A possitive number means having to wait
        
        void fire() {
            spawn(new Bomb(x, y, turretAngle, Color.RED, Bot.this));
            cooldown = cooldownTime;
        }
    }
    
    
    /** The shootgun fires short lived bullets */
    private final class Shutgun extends Weapon {
        private static final int cooldownTime = 40; // Cooldown of shutgun. Able to fire when 0. A possitive number means having to wait
        
        void fire() {
            for (int i = 0; i < 4; i++) {
                Bullet bullet = new Bullet(x + Math.cos(turretAngle - 15 + i * 10) * 20, y + Math.sin(turretAngle - 15 + i * 10) * 20, turretAngle - 15 + i * 10, Color.RED, Bot.this);
                bullet.isShortLived = true;
                bullet.firePower = 8;
                spawn(bullet);
                cooldown = cooldownTime;
            }
        }
    }
    
    
    /** The rocket launcher fires, well, rockets */
    private final class RocketLauncher extends Weapon {
        private static final int cooldownTime = 60; // Cooldown of gun. Able to fire when 0. A possitive number means having to wait
        
        void fire() {
            Rocket rocket = new Rocket(x, y, turretAngle, Color.RED, Bot.this);
            spawn(rocket);
            cooldown = cooldownTime;
        }
    }
    
    
    /** The sniper riffle fires fast, but normal, bullets */
    private final class Sniper extends Weapon {
        private static final int cooldownTime = 100; // Cooldown of gun. Able to fire when 0. A possitive number means having to wait
        
        void fire() {
            Bullet bullet = new Bullet(x, y, turretAngle, Color.RED, Bot.this);
            bullet.moveSpeed = 50;
            bullet.firePower = 100;
            spawn(bullet);
            cooldown = cooldownTime;
        }
    }
    
    
    private final void changeToPistol() {
        if (weapon != pistol) {
            weapon = pistol;
            cooldown = pistol.cooldownTime;
        }
    }
    
    
    private final void changeToBombs() {
        if (weapon != bombs) {
            weapon = bombs;
            cooldown = bombs.cooldownTime;
        }
    }
    
    
    private final void changeToRockets() {
        if (weapon != rockets) {
            weapon = rockets;
            cooldown = rockets.cooldownTime;
        }
    }
    
    
    private final void changeToSniper() {
        if (weapon != sniper) {
            weapon = sniper;
            cooldown = sniper.cooldownTime;
        }
    }
    
    
    private final void changeToMinigun() {
        if (weapon != minigun) {
            weapon = minigun;
            cooldown = minigun.cooldownTime;
        }
    }
    
    
    private final void changeToShutgun() {
        if (weapon != shutgun) {
            weapon = shutgun;
            cooldown = shutgun.cooldownTime;
        }
    }
    
    
    protected final void chooseWeapon(double weapon) {
        fireNumber = 0;
        actionQueue.clear();
        
        if (weapon == 1) {
            changeToPistol();
        } else if (weapon == 2) {
            changeToBombs();
        } else if (weapon == 3) {
            changeToRockets();
        } else if (weapon == 4) {
            changeToSniper();
        } else if (weapon == 5) {
            changeToMinigun();
        } else if (weapon == 6) {
            changeToShutgun();
        }
    }
    
    
    /** Draws the bot on the screen */
    final void draw(Graphics2D g2d) {
        // computes the angle start and extent
        if (mouth == CLOSE) {
            angleStart -= 5;
            angleExtent += 10;
        }
        if (mouth == OPEN) {
            angleStart += 5;
            angleExtent -= 10;
        }
        if (angleStart == 0)
            mouth = OPEN;
        if (angleStart > 45)
            mouth = CLOSE;
        
        // Draw the packman
        Arc2D arc = new Arc2D.Double(Arc2D.PIE);
        arc.setFrame(0, 0, width, height);
        arc.setAngleStart(angleStart);
        arc.setAngleExtent(angleExtent);
        AffineTransform at = AffineTransform.getTranslateInstance(x, y);
        at.rotate(Math.toRadians(angle));
        at.translate(-width / 2, -height / 2);
        g2d.setColor(getColor());
        g2d.fill(at.createTransformedShape(arc));
        
        // If this is not a marker
        if (isCustom()) {
            // Draw background of health bar
            RoundRectangle2D.Double rectangle = new RoundRectangle2D.Double();
            rectangle.setFrame(x - width / 2, y - height / 2 - 8, width, 5); // Outer frame
            g2d.setColor(Color.CYAN);
            g2d.fill(rectangle);
            
            // Draw status in health bar
            rectangle.setFrame(x - width / 2, y - height / 2 - 8, (width) * health / 100, 5); // Inner frame
            if (health > 30) {
                g2d.setColor(Color.BLUE);
            } else {
                g2d.setColor(Color.RED);
            }
            g2d.fill(rectangle);
            
            // Tekst below bot: Used for name of bot and tekst when speaking
            g2d.setFont(new Font("Comic Sans MS", Font.PLAIN, 12));
            g2d.setColor(Color.RED);
            g2d.drawString(this.getClass().getName(), (float) (x - width / 2), (float) (y - height / 2 - 12)); // Class name of bot
            if (sayQueue.size() > 0) {
                g2d.drawString((String) sayQueue.get(0), (float) (x - width / 2), (float) (y + height / 2 + 11));
            }
        } // End "if this is not a marker"
        
        // Draw the barrel
        RoundRectangle2D.Double barrel = new RoundRectangle2D.Double();
        barrel.setFrame(0, -5 / 2, width / 2, 5);
        AffineTransform rotateBarrel = AffineTransform.getTranslateInstance(x, y);
        rotateBarrel.rotate(Math.toRadians(turretAngle));
        at.translate(-width / 2, -height / 2);
        g2d.setColor(Color.black);
        g2d.fill(rotateBarrel.createTransformedShape(barrel));
    }
    
    
    /** Gives the name of this bot. If it is not a custom bot (i.e. it is an instance of this class used as a bot marker) don't display a name */
    public final String toString() {
        String name = "Bot marker";
        if (this.getClass().getPackage() == null)
            name = this.getClass().getName();
        return name;
    }
    
    
    /** Crude way of testing whather or not this bot is of the same type as a given bot. Used by Setup.java:jButtonCompileActionPerformed(...) */
    final boolean sameType(Bot bot) {
        if (toString() == bot.toString())
            return true;
        return false;
    }
    
    
    /** Sets angle to be between 0 and 360 degrees */
    private final double fixAngle(double angle) {
        while (angle < 0 || angle > 360) {
            if (angle > 360) angle += -360;
            if (angle < 0) angle += 360;
        }
        return angle;
    }
    
    
    /** Should be called once for each game tick. Calculates new possisions etc. */
    final void step() {
        tick++;
        
        // Calculate if a sting should be shown
        if (sayQueue.size() > 0) { // A string should be printet
            if (sayTimer == 0) // If a string has just been added
                sayTimer = sayTimerStart; // Then reset the timer
            sayTimer--; // Make the timer tick down by one
            if (sayTimer == 0) { // If the current string should stop being shown
                sayQueue.remove(0); // Remove the string
            }
        }
        
        if (isDead()) {
            height += -0.4;
            width += -0.4;
            if (height < 1 || width < 1) Game.kill(this);
        } else {
            
            if (moveQueue.isEmpty()) {
                double moveToX = x, moveToY = y;
                if (move == FORWARD) {
                    moveToX = x + Math.cos(Math.toRadians(angle)) * moveSpeed;
                    moveToY = y + Math.sin(Math.toRadians(angle)) * moveSpeed;
                } else if (move == BACKWARD) {
                    moveToX = x + Math.cos(Math.toRadians(angle)) * -moveSpeed;
                    moveToY = y + Math.sin(Math.toRadians(angle)) * -moveSpeed;
                } else if (move == RIGHT) {
                    moveToX = x + Math.cos(Math.toRadians(90 + angle)) * moveSpeed;
                    moveToY = y + Math.sin(Math.toRadians(90 + angle)) * moveSpeed;
                } else if (move == LEFT) {
                    moveToX = x + Math.cos(Math.toRadians(90 + angle)) * -moveSpeed;
                    moveToY = y + Math.sin(Math.toRadians(90 + angle)) * -moveSpeed;
                }
                moveTo(moveToX, moveToY);
                
            } else { // Then moveQueue.size() > 0
                
                if (moveDistance == 0) {
                    moveDistance = ((MoveElement) moveQueue.get(0)).distance;
                    moveBackOrForth = ((MoveElement) moveQueue.get(0)).moveBackOrForth;
                }
                
                double moveX, moveY;
                if (moveBackOrForth) {
                    moveX = Math.cos(Math.toRadians(angle)) * moveSpeed;
                    moveY = Math.sin(Math.toRadians(angle)) * moveSpeed;
                } else {
                    moveX = Math.cos(Math.toRadians(angle + 90)) * moveSpeed;
                    moveY = Math.sin(Math.toRadians(angle + 90)) * moveSpeed;
                }
                
                if (moveDistance > 0) {
                    moveTo(x + moveX, y + moveY);
                    moveDistance += - Math.sqrt(moveX * moveX + moveY * moveY);
                    
                    if (moveDistance <= 0) {
                        moveDistance = 0;
                        moveQueue.remove(0);
                    }
                    
                } else { // Then moveDistance < 0
                    moveTo(x - moveX, y - moveY);
                    moveDistance += Math.sqrt(moveX * moveX + moveY * moveY);
                    
                    if (moveDistance >= 0) {
                        moveDistance = 0;
                        moveQueue.remove(0);
                    }
                    
                }
                
            }
            
            if (rotateQueue.isEmpty()) {
                if (rotate == RIGHT) {
                    angle = fixAngle(angle + turnSpeed);
                    turretAngle = fixAngle(turretAngle + turnSpeed);
                } else if (rotate == LEFT) {
                    angle = fixAngle(angle - turnSpeed);
                    turretAngle = fixAngle(turretAngle - turnSpeed);
                }
                
            } else {
                
                if (angleAmount == 0) {
                    angleAmount = ((Double) rotateQueue.get(0)).doubleValue();
                }
                
                if (angleAmount > 0) {
                    angle = fixAngle(angle + turnSpeed);
                    turretAngle = fixAngle(turretAngle + turnSpeed);
                    
                    angleAmount += -turnSpeed;
                    if (angleAmount <= 0) {
                        angleAmount = 0;
                        rotateQueue.remove(0);
                    }
                    
                } else { // Then angleAmount < 0
                    angle = fixAngle(angle - turnSpeed);
                    turretAngle = fixAngle(turretAngle - turnSpeed);
                    
                    angleAmount += turnSpeed;
                    if (angleAmount >= 0) {
                        angleAmount = 0;
                        rotateQueue.remove(0);
                    }
                }
            }
            
            if (lookQueue.isEmpty()) {
                if (look == RIGHT) {
                    turretAngle = fixAngle(turretAngle + turnSpeed);
                } else if (look == LEFT) {
                    turretAngle = fixAngle(turretAngle - turnSpeed);
                }
                
            } else {
                if (turretAngleAmount == 0) {
                    turretAngleAmount = ((Double) lookQueue.get(0)).doubleValue();
                }
                
                if (turretAngleAmount > 0) {
                    turretAngle = fixAngle(turretAngle + turnSpeed);
                    turretAngleAmount += -turnSpeed;
                    
                    if (turretAngleAmount <= 0) {
                        turretAngleAmount = 0;
                        lookQueue.remove(0);
                    }
                    
                } else { // Then turretAngleAmount < 0
                    turretAngle = fixAngle(turretAngle - turnSpeed);
                    turretAngleAmount += turnSpeed;
                    
                    if (turretAngleAmount >= 0) {
                        turretAngleAmount = 0;
                        lookQueue.remove(0);
                    }
                }
            }
            
            
            if (!isIgnored(onSight)) {
                updateLineOfSight();
            }
            
            if (cooldown != 0)
                cooldown += -1;
            else {
                if (actionQueue.isEmpty()) {
                    if (action == FIRE) {
                        weapon.fire();
                    }
                } else {
                    if (fireNumber == 0)
                        fireNumber = ((Double) actionQueue.get(0)).intValue();
                    weapon.fire();
                    fireNumber--;
                    if (fireNumber == 0) {
                        actionQueue.remove(0);
                    }
                }
            }
        }
    }
    
    
    /** The following code for updateLineOfSight() is based on description from http://www.magic-software.com/Resources/TechPosts/IntersectEllipseLine.Apr2003.txt
     *
     *
     * To clarify the notation, let the line segment
     * have end points (u1,v1) and (u2,v2).  The ellipse
     * center is (x1,y1).  An end point of the major
     * axis is (x2,y2).  An end point of the minor axis
     * is (x3,y3).
     *
     * The half-length for the major axis is
     * a = sqrt((x2-x1)^2+(y2-y1)^2)
     * where the notation v^2 means v*v.  The
     * half-length for the minor axis is
     * b = sqrt((x3-x1)^2+(y3-y1)^2)
     * The angle formed by (x2-x1,y2-y1) and the
     * positive x-axis is named T.  Define c = cos(T)
     * and s = sin(T).  You can choose this angle using
     * the standard math library call
     * T = atan2(y2-y1,x2-x1)
     *
     * A point (x,y) is on the ellipse whenever
     * k1*(x-x1)^2 + k2*(x-x1)*(y-y1) + k3(y-y1)^2 - 1 = 0
     * where
     * k1 = (c/a)^2 + (s/b)^2
     * k2 = 2*s*c*(1/a^2 - 1/b^2)
     * k3 = (s/a)^2 + (c/b)^2
     *
     * The line segment is parameterized by
     * x(t) = u1 + t*(u2-u1)
     * y(t) = v1 + t*(v2-v1)
     * for 0 <= t <= 1.  Replace these in the ellipse
     * equation and rearrange to obtain a quadratic equation
     * q(t) = q0 + q1*t + q2*t^2
     * where
     * q0 = k1*(u1-x1)^2 + k2*(u1-x1)*(v1-y1)
     * + k3*(v1-y1)^2 - 1
     * q1 = 2*k1*(u2-u1)*(u1-x1) + k2*(u2-u1)*(v1-y1)
     * + k2*(v2-v1)*(u1-x1) + 2*k3*(v2-v1)*(v1-y1)
     * q2 = k1*(u2-u1)^2 + k2*(u2-u1)*(v2-v1)
     * + k3*(v2-v1)^2
     *
     * Compute the roots to q(t) = 0.
     * (1) If q1^2 < 4*q0*q2, the roots are complex valued.
     * The line containing the line segment does not
     * intersect the ellipse, so the line segment does
     * not intersect the ellipse.
     * (2) If q1^2 = 4*q0*q2, there is one real-valued root
     * t0.  The line containing the line segment is
     * tangent to the ellipse at (x(t0),y(t0)).  If
     * 0 <= t0 <= 1, then your line segment is tangent
     * to the ellipse at that point.  If t0 < 0 or
     * t0 > 1, the line segment does not intersect
     * the ellipse.
     * (3) If q1^2 > 4*q0*q2, there are two distinct
     * real-valued roots, t0 and t1.  If 0 <= t0 <= 1,
     * then the line segment intersects the ellipse
     * at (x(t0),y(t0)).  If 0 <= t1 <= 1, then the
     * line segment intersects the ellipse at
     * (x(t1),y(t1)).
     *
     */
    private final void updateLineOfSight() {
        double u1 = x,           // Line 1. point x (center of eye of bot)
        v1 = y,                  // Line 1. point y (center of eye of bot)
        u2 = x + Math.cos(Math.toRadians(turretAngle)), // Line 2. point x (how far the bot can see). Apparently this works fine with a very short line segment - setting this longer won't really make any differens in the end (well maybe - BUG?)
        v2 = y + Math.sin(Math.toRadians(turretAngle)), // Line 2. point y (how far the bot can see). Apparently this works fine with a very short line segment - setting this longer won't really make any differens in the end (well maybe - BUG?)
        x1,                      // Ellipse center x
        y1,                      // Ellipse center y
        a,                       // Ellipse 1. radius
        b,                       // Ellipse 2. radius
        T,                       // Ellipse angle formed between rotated top point and the x-axis
        c,                       // Ellipse cos(T)
        s,                       // Ellipse sin(T)
        k1,                      // Ellipse 1. coefficient
        k2,                      // Ellipse 2. coefficient
        k3,                      // Ellipse 3. coefficient
        q0,                      // Ellipse, with x and y replaced with line equation, 1. coefficient
        q1,                      // Ellipse, with x and y replaced with line equation, 2. coefficient
        q2,                      // Ellipse, with x and y replaced with line equation, 3. coefficient
        D;                       // Ellipse, with x and y replaced with line equation, Determinant
        
        double oldDistance = 999999999; // Used to calculate if new object is closer than the old one
        
        ArrayList objects = Game.getObjects();
        DefaultObject testLineOfSight;
        DefaultObject newLineOfSight = null;
        
        for (int i = 0; i < objects.size(); i++) {
            testLineOfSight = (DefaultObject) objects.get(i);
            if (testLineOfSight != this && !testLineOfSight.isBullet()) { // Don't look at itself, nor at bullets
                x1 = testLineOfSight.x; // Ellipse center x
                y1 = testLineOfSight.y; // Ellipse center y
                //a = testLineOfSight.height / 2; // This is commementet out until collision can work on ellipses
                //b = testLineOfSight.width / 2;  // This is commementet out until collision can work on ellipses
                //T = Math.toRadians(testLineOfSight.angle -90);  // This is commementet out until collision can work on ellipses
                a = testLineOfSight.height > testLineOfSight.width ? testLineOfSight.height / 2 : testLineOfSight.width / 2; // Temporary hack before collision can work on ellipses
                b = testLineOfSight.height < testLineOfSight.width ? testLineOfSight.height / 2 : testLineOfSight.width / 2; // Temporary hack before collision can work on ellipses
                T = Math.atan2(testLineOfSight.height > testLineOfSight.width ? testLineOfSight.height / 2 : 0, testLineOfSight.height > testLineOfSight.width ? 0 : testLineOfSight.width / 2); // Temporary hack before collision can work on ellipses
                c = Math.cos(T);
                s = Math.sin(T);
                
                k1 = (c/a)*(c/a) + (s/b)*(s/b);
                k2 = 2*s*c*(1/(a*a) - 1/(b*b));
                k3 = (s/a)*(s/a) + (c/b)*(c/b);
                
                q0 = k1*(u1-x1)*(u1-x1) + k2*(u1-x1)*(v1-y1) + k3*(v1-y1)*(v1-y1) - 1;
                q1 = 2*k1*(u2-u1)*(u1-x1) + k2*(u2-u1)*(v1-y1) + k2*(v2-v1)*(u1-x1) + 2*k3*(v2-v1)*(v1-y1);
                q2 = k1*(u2-u1)*(u2-u1) + k2*(u2-u1)*(v2-v1) + k3*(v2-v1)*(v2-v1);
                
                D = q1*q1 - 4*q0*q2;
                if (D >= 0) { // The line intersects the ellipse (if == the line is tangent, if < they are seperated)
                    
                    double t0 = (- q1 + Math.sqrt(D)) / (2 * q0);
                    double t1 = (- q1 - Math.sqrt(D)) / (2 * q0);
                    
                    if (t0 >= 0 || t1 >= 0 ) {
                        
                        // Check if the target (testLineOfSight) is closer than the old one (newLineOfSight)
                        double newDistance = 99999999;
                        
                        if (t0 >= 0) {
                            double xt0 = u1 + t0*(u2-u1);
                            double yt0 = v1 + t0*(v2-v1);
                            newDistance = Math.sqrt((x - xt0) * (x - xt0) + (y - yt0) * (y - yt0));
                        }
                        if (t1 >= 0) {
                            double xt1 = u1 + t1*(u2-u1);
                            double yt1 = v1 + t1*(v2-v1);
                            double Distance2 = Math.sqrt((x - xt1) * (x - xt1) + (y - yt1) * (y - yt1));
                            if (newDistance > Distance2)
                                newDistance = Distance2;
                        }
                        
                        // FIXME: Jeg fatter s ikke hvorfor det virker (bedre) med oldDistance < newDistance. Giver faktisk meget lidt mening nu man tnker over det - men det ser ud til at virke s what the hey...
                        if (newLineOfSight == null || oldDistance < newDistance) { // If new target is closer than the old one
                            newLineOfSight = testLineOfSight;
                            oldDistance = newDistance;
                        }
                    }
                }
            }
        }
        
        if (objectInSight != newLineOfSight) {
            objectInSight = newLineOfSight;
            throwEventOnSight(newLineOfSight);
        }
        
        if (newLineOfSight == null || objectInSight == null || objectInSight == this) Setup.notice("BUG IN UpdateLineOfSight! (object: " + this + " tick: " + tick);
        
    }
    
    
    /** Get the viewing angle as compared to the movement angle */
    protected final double getViewingAngle() {
        return turretAngle - angle;
    }
    
    
    /** Get the viewing angle as compared to the movement angle */
    protected final double getViewingDistance() {
        updateLineOfSight();
        return Math.sqrt((x - objectInSight.getX()) * (x - objectInSight.getX()) + (y - objectInSight.getY()) * (y - objectInSight.getY()));
    }
    
    
    /** This defines how to loose health */
    final void looseHealth(double firePower) {
        health += - firePower;
        if (health <= 0) {
            die();
        }
    }
    
    
    /** Let the bot die. Step() will remove the bot from the game when time comes */
    final void die() {
        isDead = true;
        isGhost = true;
    }
    
    
    /** Print a small messege just below the bot */
    protected final void say(String say) {
        sayQueue.add(say);
    }
    
    
    /** Fires the currently held gun continuouly */
    protected final void shoot() {
        action = FIRE;
        fireNumber = 0;
        actionQueue.clear();
    }
    
    
    /** Moves forward continuouly */
    protected final void moveForward() {
        move = FORWARD;
        moveDistance = 0;
        moveQueue.clear();
    }
    
    
    /** Moves backward continuouly */
    protected final void moveBackward() {
        move = BACKWARD;
        moveDistance = 0;
        moveQueue.clear();
    }
    
    
    /** Slides left continuouly */
    protected final void moveLeft() {
        move = LEFT;
        moveDistance = 0;
        moveQueue.clear();
    }
    
    
    /** Slides right continuouly */
    protected final void moveRight() {
        move = RIGHT;
        moveDistance = 0;
        moveQueue.clear();
    }
    
    
    /** Turn right continuouly */
    protected final void rotateRight() {
        rotate = RIGHT;
        angleAmount = 0;
        rotateQueue.clear();
    }
    
    
    /** Turn left continuouly */
    protected final void rotateLeft() {
        rotate = LEFT;
        angleAmount = 0;
        rotateQueue.clear();
    }
    
    
    /** Look right continuouly */
    protected final void lookRight() {
        look = RIGHT;
        turretAngleAmount = 0;
        lookQueue.clear();
    }
    
    
    /** Look left continuouly */
    protected final void lookLeft() {
        look = LEFT;
        turretAngleAmount = 0;
        lookQueue.clear();
    }
    
    
    /** Look to a specified angle */
    protected final void look(double newAngle) {
        lookQueue.add(new Double(newAngle)); // Look specified in angles
        look = STOP; // Don't look in a direction
    }
    
    
    /** Rotate to a specified angle */
    protected final void rotate(double newAngle) {
        rotateQueue.add(new Double(newAngle));
        rotate = STOP;
    }
    
    
    /** Move forward or backward for a spedified distance.
     * Note that this is not given in the usual messurements, but the input will be
     * transformed to those by calculating how many "action-ticks" it will take.
     * An action-tick is defined as the time it takes to rotate 1 degree.
     * This is only to make sleep(int) work as it should */
    /*protected final void move(double actionDistance) {
        moveQueue.add(new Double(actionDistance * moveSpeed / turnSpeed));
        move = STOP;
    }*/
    
    
    /** Move forward or backward for a spedified distance */
    protected final void move(double distance) {
        moveQueue.add(new MoveElement(distance, true));
        move = STOP;
    }
    
    
    /** Move forward or backward for a spedified distance */
    protected final void strafe(double distance) {
        moveQueue.add(new MoveElement(distance, false));
        move = STOP;
    }
    
    
    /** Fire a specified number of bullets */
    protected final void shoot(double bullets) {
        actionQueue.add(new Double(bullets));
        action = STOP;
    }
    
    
    /** Stop movement */
    protected final void stopMove() {
        move = STOP;
        moveDistance = 0;
        moveQueue.clear();
    }
    
    
    /** Stop rotating */
    protected final void stopRotate() {
        rotate = STOP;
        angleAmount = 0;
        rotateQueue.clear();
    }
    
    
    /** Stop looking */
    protected final void stopLook() {
        look = STOP;
        turretAngleAmount = 0;
        lookQueue.clear();
    }
    
    /** Stop shooting */
    protected final void stopShoot() {
        action = STOP;
        fireNumber = 0;
        actionQueue.clear();
    }
    
    /** Stop all actions */
    protected final void stop() {
        stopMove();
        stopRotate();
        stopLook();
        stopShoot();
    }
    
    
    /** Make the bot sleep for a number of action-ticks */
    /*protected final void sleep(double actionTicks) {
        int oldTick = tick;
        while (tick - oldTick < actionTicks / turnSpeed) {}
    }*/
    
    
    /*protected final void wait(int ticks) {
        int oldTick = tick;
        while (tick - oldTick < ticks) {}
    }*/
    
    
    protected final void waitForEvents() {
        while (true) {
            eventInformation.thread.yield();
        }
    }
    
    
    protected final void waitForActions() {
        waitForMove();
        waitForRotate();
        waitForLook();
        waitForShoot();
    }
    
    
    protected final void waitForMove() {
        while (!moveQueue.isEmpty()) {
            eventInformation.thread.yield();
        }
    }
    
    
    protected final void waitForRotate() {
        while (!rotateQueue.isEmpty()) {
            eventInformation.thread.yield();
        }
    }
    
    
    protected final void waitForLook() {
        while (!lookQueue.isEmpty()) {
            eventInformation.thread.yield();
        }
    }
    
    
    protected final void waitForShoot() {
        while (!actionQueue.isEmpty()) {
            eventInformation.thread.yield();
        }
    }
    
}