/*
 * Bullet.java
 *
 * Created on 19. februar 2004, 18:36
 */


package Game;

import java.awt.Color;
import java.awt.geom.Ellipse2D;
import java.awt.geom.AffineTransform;


/** A bullet that kills things :) */
final class Bullet extends DefaultObject {
    
    
    int moveSpeed = 10; // Higher means faster movement
    boolean isShortLived = false;
    int timer = 60;
    
    
    /** Creates a new instance of Bullet */
    Bullet(double x, double y, double a, Color color, DefaultObject spawnedBy) {
        super(x, y, a, 5, 5, color);
        firePower = 25;
        this.spawnedBy = spawnedBy;
        onTeam = spawnedBy.onTeam;
        health = 1;
        ignore(onSight, onStart, onNoEvent, onDeath, onTargetHit);
    }
    
    
    protected final void draw(java.awt.Graphics2D g2d) {
        Ellipse2D.Double circle = new Ellipse2D.Double();
        circle.setFrame(0, 0, getWidth(), getHeight());
        AffineTransform at = AffineTransform.getTranslateInstance(x, y);
        at.rotate(Math.toRadians(angle));
        at.translate(-width / 2, -height / 2);
        g2d.setColor(getColor());
        g2d.fill(at.createTransformedShape(circle));
    }
    
    
    protected void step() {
        if (isDead()) {
            height--;
            width--;
            if (height < 1 || width < 1) Game.kill(this);
        } else {
            double missingMovement = moveSpeed;
            double move = missingMovement;
            while (missingMovement > 0) {
                if (move > 5) { // Break movement in more parts
                    move = 5;
                }
                double moveToX, moveToY;
                moveToX = x + Math.cos(Math.toRadians(angle)) * move;
                moveToY = y + Math.sin(Math.toRadians(angle)) * move;
                moveTo(moveToX, moveToY);
                missingMovement += -move;
                move = missingMovement;
                game.repaint();
            }
            
            if (isShortLived) {
                timer--;
                if (timer < 0) {
                    isDead = true;
                    isGhost = true;
                }
            }
        }
    }
    
    
    protected void onCollision() {
        isDead = true;
        isGhost = true;
    }
    
    
    protected void onHit() {
        Setup.notice("BULLET WAS HIT!");
        isDead = true;
        isGhost = true;
    }
    
    
    protected void die() { // Don't die
    }
    
}
