/*
 * Rocket.java
 *
 * Created on 12. april 2004, 23:04
 */


package Game;

import java.awt.Color;
import java.awt.geom.Ellipse2D;


/** A rocket accelerates forward until collision, where it will detonate like the bomb (but with a smaller explosion) */
public class Rocket extends DefaultObject {
    
    
    private int timer = 120; // Ticks down. 0 == KABOOM;
    private double size = 110; // The size of the explosion
    private float alpha = 1; // Color of the explosion
    double startFirePower = 1.4;
    double moveSpeed = 1.8;
    private double acceleration = 0.08;
    
    
    /** Creates a new instance of Rocket */
    Rocket(double x, double y, double angle, Color color, DefaultObject spawnedBy) {
        super(x, y, angle, 15, 15, color);
        this.spawnedBy = spawnedBy;
        onTeam = spawnedBy.onTeam;
        firePower = startFirePower;
        health = 10;
        ignore(onSight, onStart, onNoEvent, onDeath, onTargetHit);
    }
    
    
    protected final void onHit() {
        moveSpeed = 0;
        timer = 0; // Detonate if hit by a bullet or another explosion
        spawnedBy = null;
        ignore(onHit);
        ignore(onCollision);
    }
    
    
    protected final void onCollision() {
        moveSpeed = 0;
        timer = 0;
        spawnedBy = null;
        ignore(onHit);
        ignore(onCollision);
    }
    
    
    protected final void draw(java.awt.Graphics2D g2d) {
        Ellipse2D.Double circle = new Ellipse2D.Double();
        circle.setFrame(x - width / 2, y - height / 2, getWidth(), getHeight());
        g2d.setColor(new Color(1.0f, 0f, 0f, alpha));
        g2d.fill(circle);
    }
    
    
    protected void step() {
        if (isGhost()) {
            alpha += -0.05;
            if (alpha < 0) alpha = 0;
            
            firePower = alpha / startFirePower;
            if (firePower < 0) firePower = 0;
            if (alpha <= 0) Game.kill(this);
            moveTo(x, y);
        } else {
            if (timer > 0) {
                moveSpeed += acceleration;
                acceleration = acceleration * 1.16;
                
                double missingMovement = moveSpeed;
                double move = missingMovement;
                while (missingMovement > 0) {
                    if (move > 5) { // Break movement in more parts
                        move = 5;
                    }
                    double moveToX, moveToY;
                    moveToX = x + Math.cos(Math.toRadians(angle)) * move;
                    moveToY = y + Math.sin(Math.toRadians(angle)) * move;
                    moveTo(moveToX, moveToY);
                    missingMovement += -move;
                    move = missingMovement;
                    game.repaint();
                }
            } else {
                rememberSpawnedBy = spawnedBy;
                spawnedBy = null;
                height += 15;
                width += 15;
                if (height > size) {
                    isGhost = true;
                }
                moveTo(x, y);
            }
        }
    }
}
