/*
 * Setup.java
 *
 * Created on 10. marts 2004, 22:03
 */


package Game;

import org.netbeans.lib.awtextra.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import java.net.*;
import java.io.*;
import java.security.*;
import java.util.ArrayList;
import java.nio.*;
import java.nio.channels.*;
import java.nio.charset.*;


/** This is a graphical window to setup the game */
final class Setup extends JFrame {
    
    
    private static Game game;
    
    private DefaultListModel dlmAvailableBots = new DefaultListModel();
    private DefaultListModel dlmBotsInGame = new DefaultListModel();
    private DefaultListModel dlmArenas = new DefaultListModel();
    private DefaultListModel dlmObjects = new DefaultListModel();
    private DefaultTableModel dtmTable = new DefaultTableModel();
    
    private static final int spinnerStep = 10;
    
    private SpinnerNumberModel modelGameSpeed = new SpinnerNumberModel(0, 0, 200, 1);
    private SpinnerNumberModel modelArenaWidth = new SpinnerNumberModel(500, 100, 5000, spinnerStep);
    private SpinnerNumberModel modelArenaHeight = new SpinnerNumberModel(500, 100, 5000, spinnerStep);
    private SpinnerNumberModel modelObstacleWidth = new SpinnerNumberModel(500, 1, 5000, spinnerStep);
    private SpinnerNumberModel modelObstacleHeight = new SpinnerNumberModel(500, 1, 5000, spinnerStep);
    private SpinnerNumberModel modelX = new SpinnerNumberModel(100, 5, 5000, spinnerStep);
    private SpinnerNumberModel modelY = new SpinnerNumberModel(100, 5, 5000, spinnerStep);
    private SpinnerNumberModel modelAngle = new SpinnerNumberModel(0, 0, 360, 10);
    private SpinnerNumberModel modelTimer = new SpinnerNumberModel(2000, 50, 9999, 100);
    
    private Component oldComponentFocus = null;
    private ArrayList removedArenas = new ArrayList();
    
    
    /** Creates new form Setup */
    Setup(Game game) {
        initComponents();
        
        this.game = game;
        
        jListAvailableBots.setModel(dlmAvailableBots);
        jListBotsInGame.setModel(dlmBotsInGame);
        jListArenas.setModel(dlmArenas);
        jListObjects.setModel(dlmObjects);
        
        modelGameSpeed.setValue(new Integer(game.gameSpeed));
        jSpinnerGameSpeed.setModel(modelGameSpeed);
        
        jSpinnerArenaWidth.setModel(modelArenaWidth);
        jSpinnerArenaHeight.setModel(modelArenaHeight);
        jSpinnerObstacleWidth.setModel(modelObstacleWidth);
        jSpinnerObstacleHeight.setModel(modelObstacleHeight);
        jSpinnerX.setModel(modelX);
        jSpinnerY.setModel(modelY);
        jSpinnerAngle.setModel(modelAngle);
        jSpinnerTimeout.setModel(modelTimer);
        
        jCheckBoxFriendlyFire.setSelected(game.isDamageOnFriendlyFire);
        jCheckBoxNiceRendering.setSelected(game.usingNiceRendering);
        jCheckBoxRestartRound.setSelected(game.willStartNewRoundAfterThis);
        jCheckBoxTimer.setSelected(game.timerIsEnabled);
        jCheckBoxTeamplay.setSelected(game.isTeamPlayOn);
        jLabelNoteTeamplay.setVisible(!jCheckBoxTeamplay.isSelected());
        
        dtmTable.addColumn("Name");
        dtmTable.addColumn("Kills");
        dtmTable.addColumn("Deaths");
        dtmTable.addColumn("Wins");
        dtmTable.addColumn("Pain inflicted");
        jTableStatistics.setModel(dtmTable);
        
        listBots();
        listArenas();
        
        setVisible(true);
    }
    
    
    void gameEnded() {
        jButtonStartGame.setText("Start game");
        jButtonStopGame.setEnabled(false);
        jButtonEndRound.setEnabled(false);
        
        if (jListArenas.getSelectedIndex() != -1) {
            Arena arena = (Arena) jListArenas.getSelectedValue();
            arena.spawnAll();
        }
    }
    
    
    void startGame() {
        jButtonStartGame.setText("Pause");
        jButtonStopGame.setEnabled(true);
        if (jCheckBoxRestartRound.isSelected())
            jButtonEndRound.setEnabled(true);
    }
    
    
    static void notice(String notice) {
        javax.swing.JOptionPane.showConfirmDialog(null, notice, "Notice", javax.swing.JOptionPane.CLOSED_OPTION);
    }
    
    
    static void error(Exception e) {
        javax.swing.JOptionPane.showConfirmDialog(null, "An " + e.getClass() + " exception has been caught.\nThe exception message was:\n" + e.getMessage(), "Exception", javax.swing.JOptionPane.CLOSED_OPTION);
        System.out.println(e);
    }
    
    
    static void error(String error, Exception e) {
        javax.swing.JOptionPane.showConfirmDialog(null, "An " + e.getClass() + " exception has been caught.\n" + error + "\nThe exception message was:\n" + e.getMessage(), "Exception", javax.swing.JOptionPane.CLOSED_OPTION);
        System.out.println(e);
    }
    
    
    private void listBots() {
        // Get the location of this class
        Class cls = this.getClass();
        ProtectionDomain pDomain = cls.getProtectionDomain();
        CodeSource cSource = pDomain.getCodeSource();
        URL botDir = cSource.getLocation();
        File botFileDir = new File(botDir.getPath() + File.separator + "Bots" + File.separator);
        
        try {
            botDir = botFileDir.toURL();
        } catch (MalformedURLException e) {
            error(e);
        }
        
        URL[] botDirs = new URL[]{botDir};
        
        // Create a new class loader with the directory
        ClassLoader cl = new URLClassLoader(botDirs);
        
        java.io.File[] files = botFileDir.listFiles();
        listBotsFromFiles(files, cl);
    }
    
    private void listBotsFromFiles(File[] files, ClassLoader cl) {
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                if (files[i].getAbsolutePath().endsWith(".class")) {
                    Bot bot = null;
                    Class botClass = null;
                    // Try to load the bot
                    try {
                        botClass = cl.loadClass(files[i].getName().substring(0, files[i].getName().length() - 6));
                        bot = (Bot) botClass.newInstance();
                    } catch (ClassCastException e) {
                        error("Ignoring the not-a-bot: " + files[i].getName().substring(0, files[i].getName().length() - 6) + " because it is not extending from Game.Bot (and if it actually isn't a bot, don't be a wanker and place it in the bot-directory!)\n If you think this is a bug, the exception message was:", e);
                    } catch (ClassNotFoundException e) {
                        error("Ignoring the not-a-bot: " + files[i].getName().substring(0, files[i].getName().length() - 6) + " because it is not extending from Game.Bot (and if it actually isn't a bot, don't be a wanker and place it in the bot-directory!)\n If you think this is a bug, the exception message was:", e);
                    } catch (InstantiationException e) {
                        error("Ignoring the not-a-bot: " + files[i].getName().substring(0, files[i].getName().length() - 6) + " because it is not extending from Game.Bot (and if it actually isn't a bot, don't be a wanker and place it in the bot-directory!)\n If you think this is a bug, the exception message was:", e);
                    } catch (IllegalAccessException e) {
                        error("Ignoring the not-a-bot: " + files[i].getName().substring(0, files[i].getName().length() - 6) + " because it is not extending from Game.Bot (and if it actually isn't a bot, don't be a wanker and place it in the bot-directory!)\n If you think this is a bug, the exception message was:", e);
                    }
                    
                    if (bot != null) {
                        dlmAvailableBots.addElement(botClass);
                    }
                }
            }
        }
        
        if (dlmAvailableBots.getSize() > 0)
            jListAvailableBots.setSelectedIndex(0);
    }
    
    
    private void listArenas() {
        // Get the location of this class
        Class cls = this.getClass();
        ProtectionDomain pDomain = cls.getProtectionDomain();
        CodeSource cSource = pDomain.getCodeSource();
        URL arenaDir = cSource.getLocation();
        File arenaFileDir = new File(arenaDir.getPath() + File.separator + "Arenas" + File.separator);
        java.io.File[] files = arenaFileDir.listFiles();
        listArenasFromFiles(files);
    }
    
    
    private void listArenasFromFiles(File[] files) {
        if (files != null) {
            for (int i = 0; i < files.length; i++) {
                if (files[i].getAbsolutePath().endsWith(".arena")) {
                    Arena arena = new Arena();
                    if (arena.load(files[i]))
                        dlmArenas.addElement(arena);
                }
            }
        }
        if (dlmArenas.getSize() > 0)
            jListArenas.setSelectedIndex(0);
    }
    
    
    void statsAddKill(Bot bot) {
        // Find the bot in the table
        for (int i = 0; i < dtmTable.getRowCount(); i++) {
            int j = dtmTable.findColumn("Name");
            int k = dtmTable.findColumn("Kills");
            if (((Arena.FakeBot) dtmTable.getValueAt(i, j)).real == bot) {
                dtmTable.setValueAt(new Integer(((Integer) dtmTable.getValueAt(i, k)).intValue() + 1), i, k); // Add 1 to the current value
                break;
            } else if (i == dtmTable.getRowCount() - 1)
                notice("BUG in jButtonRemoveBotActionPerformed!");
        }
    }
    
    
    void statsAddDeath(Bot bot) {
        // Find the bot in the table
        for (int i = 0; i < dtmTable.getRowCount(); i++) {
            int j = dtmTable.findColumn("Name");
            int k = dtmTable.findColumn("Deaths");
            if (((Arena.FakeBot) dtmTable.getValueAt(i, j)).real == bot) {
                dtmTable.setValueAt(new Integer(((Integer) dtmTable.getValueAt(i, k)).intValue() + 1), i, k); // Add 1 to the current value
                break;
            } else if (i == dtmTable.getRowCount() - 1)
                notice("BUG in jButtonRemoveBotActionPerformed!");
        }
    }
    
    
    void statsAddWin(Bot bot) {
        // Find the bot in the table
        for (int i = 0; i < dtmTable.getRowCount(); i++) {
            int j = dtmTable.findColumn("Name");
            int k = dtmTable.findColumn("Wins");
            if (((Arena.FakeBot) dtmTable.getValueAt(i, j)).real == bot) {
                dtmTable.setValueAt(new Integer(((Integer) dtmTable.getValueAt(i, k)).intValue() + 1), i, k); // Add 1 to the current value
                break;
            } else if (i == dtmTable.getRowCount() - 1)
                notice("BUG in jButtonRemoveBotActionPerformed!");
        }
    }
    
    
    void statsAddPain(Bot bot, double pain) {
        // Find the bot in the table
        for (int i = 0; i < dtmTable.getRowCount(); i++) {
            int j = dtmTable.findColumn("Name");
            int k = dtmTable.findColumn("Pain inflicted");
            if (((Arena.FakeBot) dtmTable.getValueAt(i, j)).real == bot) {
                dtmTable.setValueAt(new Double(((Double) dtmTable.getValueAt(i, k)).doubleValue() + pain), i, k); // Add pain to the current value
                break;
            } else if (i == dtmTable.getRowCount() - 1)
                notice("BUG in jButtonRemoveBotActionPerformed!");
        }
    }
    
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        buttonGroupShape = new javax.swing.ButtonGroup();
        jTabbedPaneSetup = new javax.swing.JTabbedPane();
        jPanelGame = new javax.swing.JPanel();
        jButtonStartGame = new javax.swing.JButton();
        jLabelGame = new javax.swing.JLabel();
        jSeparatorGame = new javax.swing.JSeparator();
        jButtonStopGame = new javax.swing.JButton();
        jSpinnerGameSpeed = new javax.swing.JSpinner();
        jLabelGameSpeed = new javax.swing.JLabel();
        jCheckBoxFriendlyFire = new javax.swing.JCheckBox();
        jCheckBoxNiceRendering = new javax.swing.JCheckBox();
        jButtonExit = new javax.swing.JButton();
        jCheckBoxTimer = new javax.swing.JCheckBox();
        jLabelTimeout = new javax.swing.JLabel();
        jSpinnerTimeout = new javax.swing.JSpinner();
        jCheckBoxRestartRound = new javax.swing.JCheckBox();
        jButtonEndRound = new javax.swing.JButton();
        jCheckBoxTeamplay = new javax.swing.JCheckBox();
        jPanelBots = new javax.swing.JPanel();
        jLabelBots = new javax.swing.JLabel();
        jSeparatorBots = new javax.swing.JSeparator();
        jScrollPaneAvailableBots = new javax.swing.JScrollPane();
        jListAvailableBots = new javax.swing.JList();
        jLabelAvailableBots = new javax.swing.JLabel();
        jButtonRemoveBot = new javax.swing.JButton();
        jButtonAddBot = new javax.swing.JButton();
        jButtonSearchForBots = new javax.swing.JButton();
        jLabelBotsInGame = new javax.swing.JLabel();
        jScrollPaneBotsInGame = new javax.swing.JScrollPane();
        jListBotsInGame = new javax.swing.JList();
        jLabelOnTeam = new javax.swing.JLabel();
        jComboBoxOnTeam = new javax.swing.JComboBox();
        jLabelNoteTeamplay = new javax.swing.JLabel();
        jPanelArena = new javax.swing.JPanel();
        jLabelLayouts = new javax.swing.JLabel();
        jButtonSaveAll = new javax.swing.JButton();
        jLabelObstacles = new javax.swing.JLabel();
        jButtonAddObstacle = new javax.swing.JButton();
        jButtonRemoveObstacle = new javax.swing.JButton();
        jScrollPaneLayouts = new javax.swing.JScrollPane();
        jListArenas = new javax.swing.JList();
        jScrollPaneObstacles = new javax.swing.JScrollPane();
        jListObjects = new javax.swing.JList();
        jLabelArena = new javax.swing.JLabel();
        jSeparatorArena = new javax.swing.JSeparator();
        jLabelWidth = new javax.swing.JLabel();
        jLabelHeight = new javax.swing.JLabel();
        jButtonSearchForLayout = new javax.swing.JButton();
        jButtonNewLayout = new javax.swing.JButton();
        jButtonRemoveLayout = new javax.swing.JButton();
        jLabelObstacleWidth = new javax.swing.JLabel();
        jLabelObstacleHeight = new javax.swing.JLabel();
        jLabelY = new javax.swing.JLabel();
        jLabelX = new javax.swing.JLabel();
        jSpinnerY = new javax.swing.JSpinner();
        jSpinnerAngle = new javax.swing.JSpinner();
        jSpinnerObstacleHeight = new javax.swing.JSpinner();
        jSpinnerArenaWidth = new javax.swing.JSpinner();
        jLabelName = new javax.swing.JLabel();
        jTextFieldName = new javax.swing.JTextField();
        jSpinnerObstacleWidth = new javax.swing.JSpinner();
        jSpinnerArenaHeight = new javax.swing.JSpinner();
        jButtonAddBotPosition = new javax.swing.JButton();
        jButtonRemoveBotPosition = new javax.swing.JButton();
        jSpinnerX = new javax.swing.JSpinner();
        jLabelAngle = new javax.swing.JLabel();
        jPanelStatistics = new javax.swing.JPanel();
        jLabelStatistics = new javax.swing.JLabel();
        jSeparatorStatistics = new javax.swing.JSeparator();
        jScrollPaneStatistics = new javax.swing.JScrollPane();
        jTableStatistics = new javax.swing.JTable();
        jButton1 = new javax.swing.JButton();
        jPanelCustomBot = new javax.swing.JPanel();
        jLabelCustomBot = new javax.swing.JLabel();
        jSeparatorCustomBot = new javax.swing.JSeparator();
        jButtonCompile = new javax.swing.JButton();
        jButtonSave = new javax.swing.JButton();
        jButtonLoad = new javax.swing.JButton();
        jScrollPaneCustomBot = new javax.swing.JScrollPane();
        jTextAreaCustomBot = new javax.swing.JTextArea();
        jTextFieldCustomBotName = new javax.swing.JTextField();
        jLabel1 = new javax.swing.JLabel();
        jPanelHelp = new javax.swing.JPanel();
        jSeparatorHelp = new javax.swing.JSeparator();
        jLabelWelcome = new javax.swing.JLabel();
        jScrollPaneHelp = new javax.swing.JScrollPane();
        jTextAreaHelp = new javax.swing.JTextArea();

        getContentPane().setLayout(new java.awt.FlowLayout());

        setTitle("Setup");
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jTabbedPaneSetup.setPreferredSize(new java.awt.Dimension(405, 405));
        jTabbedPaneSetup.setMinimumSize(new java.awt.Dimension(405, 405));
        jTabbedPaneSetup.setDebugGraphicsOptions(javax.swing.DebugGraphics.NONE_OPTION);
        jTabbedPaneSetup.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jTabbedPaneSetupStateChanged(evt);
            }
        });

        jPanelGame.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jButtonStartGame.setText("Start game");
        jButtonStartGame.setEnabled(false);
        jButtonStartGame.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonStartGameActionPerformed(evt);
            }
        });

        jPanelGame.add(jButtonStartGame, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 220, 100, -1));

        jLabelGame.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 18));
        jLabelGame.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelGame.setText("Game");
        jPanelGame.add(jLabelGame, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 10, 400, -1));

        jPanelGame.add(jSeparatorGame, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 400, 10));

        jButtonStopGame.setText("Stop game");
        jButtonStopGame.setEnabled(false);
        jButtonStopGame.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonStopGameActionPerformed(evt);
            }
        });

        jPanelGame.add(jButtonStopGame, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 250, 100, -1));

        jSpinnerGameSpeed.setToolTipText("Higher means slower game play");
        jSpinnerGameSpeed.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerGameSpeedStateChanged(evt);
            }
        });

        jPanelGame.add(jSpinnerGameSpeed, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 60, 50, 20));

        jLabelGameSpeed.setText("Game speed:");
        jPanelGame.add(jLabelGameSpeed, new org.netbeans.lib.awtextra.AbsoluteConstraints(210, 60, -1, 20));

        jCheckBoxFriendlyFire.setText("Damage on friendly fire");
        jCheckBoxFriendlyFire.setSelected(true);
        jCheckBoxFriendlyFire.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxFriendlyFireActionPerformed(evt);
            }
        });

        jPanelGame.add(jCheckBoxFriendlyFire, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 60, -1, 20));

        jCheckBoxNiceRendering.setText("Nice rendering");
        jCheckBoxNiceRendering.setToolTipText("Turn anti-aliasing on or off");
        jCheckBoxNiceRendering.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxNiceRenderingActionPerformed(evt);
            }
        });

        jPanelGame.add(jCheckBoxNiceRendering, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 90, -1, 20));

        jButtonExit.setText("Exit");
        jButtonExit.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonExitActionPerformed(evt);
            }
        });

        jPanelGame.add(jButtonExit, new org.netbeans.lib.awtextra.AbsoluteConstraints(290, 340, 100, -1));

        jCheckBoxTimer.setText("Timer");
        jCheckBoxTimer.setSelected(true);
        jCheckBoxTimer.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxTimerActionPerformed(evt);
            }
        });

        jPanelGame.add(jCheckBoxTimer, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 120, -1, -1));

        jLabelTimeout.setText("Timeout:");
        jPanelGame.add(jLabelTimeout, new org.netbeans.lib.awtextra.AbsoluteConstraints(210, 90, -1, 20));

        jSpinnerTimeout.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerTimeoutStateChanged(evt);
            }
        });

        jPanelGame.add(jSpinnerTimeout, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 90, 50, 20));

        jCheckBoxRestartRound.setText("Start a new round after this");
        jCheckBoxRestartRound.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxRestartRoundActionPerformed(evt);
            }
        });

        jPanelGame.add(jCheckBoxRestartRound, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 150, -1, -1));

        jButtonEndRound.setText("End round");
        jButtonEndRound.setEnabled(false);
        jButtonEndRound.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonEndRoundActionPerformed(evt);
            }
        });

        jPanelGame.add(jButtonEndRound, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 280, 100, -1));

        jCheckBoxTeamplay.setText("Teamplay");
        jCheckBoxTeamplay.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxTeamplayActionPerformed(evt);
            }
        });

        jPanelGame.add(jCheckBoxTeamplay, new org.netbeans.lib.awtextra.AbsoluteConstraints(30, 180, -1, 20));

        jTabbedPaneSetup.addTab("Game", jPanelGame);

        jPanelBots.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jLabelBots.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 18));
        jLabelBots.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelBots.setText("Bots");
        jPanelBots.add(jLabelBots, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 10, 400, -1));

        jPanelBots.add(jSeparatorBots, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 400, 10));

        jListAvailableBots.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListAvailableBots.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                jListAvailableBotsValueChanged(evt);
            }
        });

        jScrollPaneAvailableBots.setViewportView(jListAvailableBots);

        jPanelBots.add(jScrollPaneAvailableBots, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, 130, 180));

        jLabelAvailableBots.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelAvailableBots.setText("Available bots:");
        jPanelBots.add(jLabelAvailableBots, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, 130, 20));

        jButtonRemoveBot.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 10));
        jButtonRemoveBot.setText("<- Remove");
        jButtonRemoveBot.setEnabled(false);
        jButtonRemoveBot.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRemoveBotActionPerformed(evt);
            }
        });

        jPanelBots.add(jButtonRemoveBot, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 140, 90, -1));

        jButtonAddBot.setText("Add ->");
        jButtonAddBot.setEnabled(false);
        jButtonAddBot.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddBotActionPerformed(evt);
            }
        });

        jPanelBots.add(jButtonAddBot, new org.netbeans.lib.awtextra.AbsoluteConstraints(150, 110, 90, -1));

        jButtonSearchForBots.setText("Search for bot");
        jButtonSearchForBots.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSearchForBotsActionPerformed(evt);
            }
        });

        jPanelBots.add(jButtonSearchForBots, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 270, 130, -1));

        jLabelBotsInGame.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelBotsInGame.setText("Bots in game:");
        jPanelBots.add(jLabelBotsInGame, new org.netbeans.lib.awtextra.AbsoluteConstraints(250, 60, 130, 20));

        jListBotsInGame.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListBotsInGame.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                jListBotsInGameValueChanged(evt);
            }
        });

        jScrollPaneBotsInGame.setViewportView(jListBotsInGame);

        jPanelBots.add(jScrollPaneBotsInGame, new org.netbeans.lib.awtextra.AbsoluteConstraints(250, 80, 130, 180));

        jLabelOnTeam.setText("On team:");
        jPanelBots.add(jLabelOnTeam, new org.netbeans.lib.awtextra.AbsoluteConstraints(250, 270, -1, 20));

        jComboBoxOnTeam.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Blue", "Green" }));
        jComboBoxOnTeam.setEnabled(false);
        jComboBoxOnTeam.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBoxOnTeamActionPerformed(evt);
            }
        });

        jPanelBots.add(jComboBoxOnTeam, new org.netbeans.lib.awtextra.AbsoluteConstraints(310, 270, 70, -1));

        jLabelNoteTeamplay.setFont(new java.awt.Font("Microsoft Sans Serif", 2, 11));
        jLabelNoteTeamplay.setText("Note: Teamplay is off");
        jPanelBots.add(jLabelNoteTeamplay, new org.netbeans.lib.awtextra.AbsoluteConstraints(250, 300, 130, -1));

        jTabbedPaneSetup.addTab("Bots", jPanelBots);

        jPanelArena.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jLabelLayouts.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelLayouts.setText("Layouts:");
        jPanelArena.add(jLabelLayouts, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, 130, 20));

        jButtonSaveAll.setText("Save All");
        jButtonSaveAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSaveAllActionPerformed(evt);
            }
        });

        jPanelArena.add(jButtonSaveAll, new org.netbeans.lib.awtextra.AbsoluteConstraints(280, 110, 110, -1));

        jLabelObstacles.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelObstacles.setText("Bots & Obstacles:");
        jPanelArena.add(jLabelObstacles, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 210, 130, 20));

        jButtonAddObstacle.setText("Add obstacle");
        jButtonAddObstacle.setEnabled(false);
        jButtonAddObstacle.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddObstacleActionPerformed(evt);
            }
        });

        jPanelArena.add(jButtonAddObstacle, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 230, 110, -1));

        jButtonRemoveObstacle.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 9));
        jButtonRemoveObstacle.setText("Remove obstacle");
        jButtonRemoveObstacle.setEnabled(false);
        jButtonRemoveObstacle.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRemoveObstacleActionPerformed(evt);
            }
        });

        jPanelArena.add(jButtonRemoveObstacle, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 260, 110, 23));

        jListArenas.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListArenas.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                jListArenasValueChanged(evt);
            }
        });

        jScrollPaneLayouts.setViewportView(jListArenas);

        jPanelArena.add(jScrollPaneLayouts, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 80, 130, 110));

        jListObjects.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        jListObjects.addListSelectionListener(new javax.swing.event.ListSelectionListener() {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt) {
                jListObjectsValueChanged(evt);
            }
        });

        jScrollPaneObstacles.setViewportView(jListObjects);

        jPanelArena.add(jScrollPaneObstacles, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 230, 130, 110));

        jLabelArena.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 18));
        jLabelArena.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelArena.setText("Arena");
        jPanelArena.add(jLabelArena, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 10, 400, -1));

        jPanelArena.add(jSeparatorArena, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 400, 10));

        jLabelWidth.setText("Width:");
        jPanelArena.add(jLabelWidth, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 170, -1, 20));

        jLabelHeight.setText("Height:");
        jPanelArena.add(jLabelHeight, new org.netbeans.lib.awtextra.AbsoluteConstraints(280, 170, -1, 20));

        jButtonSearchForLayout.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 9));
        jButtonSearchForLayout.setText("Search for layout");
        jButtonSearchForLayout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSearchForLayoutActionPerformed(evt);
            }
        });

        jPanelArena.add(jButtonSearchForLayout, new org.netbeans.lib.awtextra.AbsoluteConstraints(280, 80, 110, 23));

        jButtonNewLayout.setText("New layout");
        jButtonNewLayout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonNewLayoutActionPerformed(evt);
            }
        });

        jPanelArena.add(jButtonNewLayout, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 80, 110, -1));

        jButtonRemoveLayout.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 10));
        jButtonRemoveLayout.setText("Remove layout");
        jButtonRemoveLayout.setEnabled(false);
        jButtonRemoveLayout.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRemoveLayoutActionPerformed(evt);
            }
        });

        jPanelArena.add(jButtonRemoveLayout, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 110, 110, 23));

        jLabelObstacleWidth.setText("Width:");
        jPanelArena.add(jLabelObstacleWidth, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 290, -1, 20));

        jLabelObstacleHeight.setText("Height:");
        jPanelArena.add(jLabelObstacleHeight, new org.netbeans.lib.awtextra.AbsoluteConstraints(280, 290, -1, 20));

        jLabelY.setText("Y:");
        jPanelArena.add(jLabelY, new org.netbeans.lib.awtextra.AbsoluteConstraints(280, 320, -1, 20));

        jLabelX.setText("X:");
        jPanelArena.add(jLabelX, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 320, -1, 20));

        jSpinnerY.setEnabled(false);
        jSpinnerY.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerYStateChanged(evt);
            }
        });

        jPanelArena.add(jSpinnerY, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 320, 50, -1));

        jSpinnerAngle.setEnabled(false);
        jSpinnerAngle.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerAngleStateChanged(evt);
            }
        });

        jPanelArena.add(jSpinnerAngle, new org.netbeans.lib.awtextra.AbsoluteConstraints(200, 350, 50, -1));

        jSpinnerObstacleHeight.setEnabled(false);
        jSpinnerObstacleHeight.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerObstacleHeightStateChanged(evt);
            }
        });

        jPanelArena.add(jSpinnerObstacleHeight, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 290, 50, -1));

        jSpinnerArenaWidth.setEnabled(false);
        jSpinnerArenaWidth.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerArenaWidthStateChanged(evt);
            }
        });

        jPanelArena.add(jSpinnerArenaWidth, new org.netbeans.lib.awtextra.AbsoluteConstraints(200, 170, 50, -1));

        jLabelName.setText("Name:");
        jPanelArena.add(jLabelName, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 140, -1, 20));

        jTextFieldName.setHorizontalAlignment(javax.swing.JTextField.LEFT);
        jTextFieldName.setText("Default");
        jTextFieldName.setEnabled(false);
        jTextFieldName.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldNameActionPerformed(evt);
            }
        });
        jTextFieldName.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusLost(java.awt.event.FocusEvent evt) {
                jTextFieldNameFocusLost(evt);
            }
        });
        jTextFieldName.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                jTextFieldNameKeyTyped(evt);
            }
        });

        jPanelArena.add(jTextFieldName, new org.netbeans.lib.awtextra.AbsoluteConstraints(200, 140, 190, 20));

        jSpinnerObstacleWidth.setEnabled(false);
        jSpinnerObstacleWidth.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerObstacleWidthStateChanged(evt);
            }
        });

        jPanelArena.add(jSpinnerObstacleWidth, new org.netbeans.lib.awtextra.AbsoluteConstraints(200, 290, 50, -1));

        jSpinnerArenaHeight.setEnabled(false);
        jSpinnerArenaHeight.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerArenaHeightStateChanged(evt);
            }
        });

        jPanelArena.add(jSpinnerArenaHeight, new org.netbeans.lib.awtextra.AbsoluteConstraints(320, 170, 50, -1));

        jButtonAddBotPosition.setText("Add bot");
        jButtonAddBotPosition.setEnabled(false);
        jButtonAddBotPosition.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonAddBotPositionActionPerformed(evt);
            }
        });

        jPanelArena.add(jButtonAddBotPosition, new org.netbeans.lib.awtextra.AbsoluteConstraints(280, 230, 110, -1));

        jButtonRemoveBotPosition.setText("Remove bot");
        jButtonRemoveBotPosition.setEnabled(false);
        jButtonRemoveBotPosition.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonRemoveBotPositionActionPerformed(evt);
            }
        });

        jPanelArena.add(jButtonRemoveBotPosition, new org.netbeans.lib.awtextra.AbsoluteConstraints(280, 260, 110, 23));

        jSpinnerX.setEnabled(false);
        jSpinnerX.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jSpinnerXStateChanged(evt);
            }
        });

        jPanelArena.add(jSpinnerX, new org.netbeans.lib.awtextra.AbsoluteConstraints(200, 320, 50, -1));

        jLabelAngle.setText("Angle:");
        jPanelArena.add(jLabelAngle, new org.netbeans.lib.awtextra.AbsoluteConstraints(160, 350, -1, 20));

        jTabbedPaneSetup.addTab("Arena", jPanelArena);

        jPanelStatistics.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jLabelStatistics.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 18));
        jLabelStatistics.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelStatistics.setText("Statistics");
        jPanelStatistics.add(jLabelStatistics, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 10, 400, -1));

        jPanelStatistics.add(jSeparatorStatistics, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 400, 10));

        jTableStatistics.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null},
                {null, null, null, null}
            },
            new String [] {
                "Title 1", "Title 2", "Title 3", "Title 4"
            }
        ));
        jScrollPaneStatistics.setViewportView(jTableStatistics);

        jPanelStatistics.add(jScrollPaneStatistics, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, 380, 260));

        jButton1.setText("Reset Statistics");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        jPanelStatistics.add(jButton1, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 340, 120, -1));

        jTabbedPaneSetup.addTab("Statistics", jPanelStatistics);

        jPanelCustomBot.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jLabelCustomBot.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 18));
        jLabelCustomBot.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelCustomBot.setText("Custom Bot");
        jPanelCustomBot.add(jLabelCustomBot, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 10, 400, -1));

        jPanelCustomBot.add(jSeparatorCustomBot, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 400, 10));

        jButtonCompile.setText("Save and Compile");
        jButtonCompile.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonCompileActionPerformed(evt);
            }
        });

        jPanelCustomBot.add(jButtonCompile, new org.netbeans.lib.awtextra.AbsoluteConstraints(270, 340, 120, -1));

        jButtonSave.setText("Save");
        jButtonSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonSaveActionPerformed(evt);
            }
        });

        jPanelCustomBot.add(jButtonSave, new org.netbeans.lib.awtextra.AbsoluteConstraints(140, 340, 120, -1));

        jButtonLoad.setText("Load");
        jButtonLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonLoadActionPerformed(evt);
            }
        });

        jPanelCustomBot.add(jButtonLoad, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 340, 120, -1));

        jTextAreaCustomBot.setLineWrap(true);
        jTextAreaCustomBot.setWrapStyleWord(true);
        jScrollPaneCustomBot.setViewportView(jTextAreaCustomBot);

        jPanelCustomBot.add(jScrollPaneCustomBot, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 60, 380, 240));

        jPanelCustomBot.add(jTextFieldCustomBotName, new org.netbeans.lib.awtextra.AbsoluteConstraints(140, 310, 250, -1));

        jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabel1.setText("Name:");
        jPanelCustomBot.add(jLabel1, new org.netbeans.lib.awtextra.AbsoluteConstraints(10, 310, 120, 20));

        jTabbedPaneSetup.addTab("Custom bot", jPanelCustomBot);

        jPanelHelp.setLayout(new org.netbeans.lib.awtextra.AbsoluteLayout());

        jPanelHelp.add(jSeparatorHelp, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 40, 400, 10));

        jLabelWelcome.setFont(new java.awt.Font("Microsoft Sans Serif", 0, 18));
        jLabelWelcome.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        jLabelWelcome.setText("Help");
        jPanelHelp.add(jLabelWelcome, new org.netbeans.lib.awtextra.AbsoluteConstraints(0, 10, 400, -1));

        jTextAreaHelp.setEditable(false);
        jTextAreaHelp.setText("Welcome to FixmeName.\n\nUnfortunatly, noone of us cares about writing a help section, so you a lost my friend.\n\nGo see a shrink if you really need help.\n\n\n\n\n\n\n\n\nCopyleft C210, 2004");
        jTextAreaHelp.setLineWrap(true);
        jTextAreaHelp.setWrapStyleWord(true);
        jScrollPaneHelp.setViewportView(jTextAreaHelp);

        jPanelHelp.add(jScrollPaneHelp, new org.netbeans.lib.awtextra.AbsoluteConstraints(22, 62, 350, 290));

        jTabbedPaneSetup.addTab("Help", jPanelHelp);

        getContentPane().add(jTabbedPaneSetup);

        pack();
    }//GEN-END:initComponents
    
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        for (int i = 0; i < dtmTable.getRowCount(); i++) {
            for (int j = 1; j < dtmTable.getColumnCount(); j++) {
                dtmTable.setValueAt(new Integer(0), i, j);
            }
        }
    }//GEN-LAST:event_jButton1ActionPerformed
    
    private void jButtonSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSaveActionPerformed
        if (jTextFieldCustomBotName.getText().length() == 0) {
            notice("Please write a name for your bot");
        } else {
            saveCustomBot();
        }
    }//GEN-LAST:event_jButtonSaveActionPerformed
    
    private void saveCustomBot() {
        // Find the Bots directory
        Class cls = this.getClass();
        ProtectionDomain pDomain = cls.getProtectionDomain();
        CodeSource cSource = pDomain.getCodeSource();
        URL thisDir = cSource.getLocation();
        
        File file = new File(thisDir.getPath() + File.separator + "Bots" + File.separator + jTextFieldCustomBotName.getText() + ".java");
        
        // Save the specified file to the Bots directory
        try {
            FileChannel fileChannel = new RandomAccessFile(file, "rw").getChannel();
            fileChannel.write(ByteBuffer.wrap(jTextAreaCustomBot.getText().getBytes()));
            fileChannel.close();
        } catch (FileNotFoundException e) {
            error(e);
        } catch (IOException e) {
            error(e);
        }
    }
    
    private void jButtonLoadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonLoadActionPerformed
        // Find the Bots directory
        Class cls = this.getClass();
        ProtectionDomain pDomain = cls.getProtectionDomain();
        CodeSource cSource = pDomain.getCodeSource();
        URL thisDir = cSource.getLocation();
        File botsDir = new File(thisDir.getPath() + File.separator + "Bots" + File.separator);
        
        // Make a pop-up window to choose a .java file
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setCurrentDirectory(botsDir);
        fileChooser.addChoosableFileFilter(new javax.swing.filechooser.FileFilter() {
            public boolean accept(File file) {
                return file.getName().endsWith(".java") || file.isDirectory();
            }
            public String getDescription() {
                return "*.java";
            }
        }
        );
        fileChooser.showOpenDialog(this);
        File file = fileChooser.getSelectedFile();
        
        // Load the selected file
        if (file != null) {
            try {
                FileChannel fileChannel = new RandomAccessFile(file, "rw").getChannel();
                ByteBuffer bb = fileChannel.map(FileChannel.MapMode.READ_ONLY, 0, fileChannel.size());
                String encoding = System.getProperty("file.encoding");
                //String encoding = "UTF-8";
                CharBuffer cb = Charset.forName(encoding).decode(bb);
                jTextAreaCustomBot.setText(cb.toString());
                jTextFieldCustomBotName.setText(file.getName().substring(0, file.getName().length() - 5)); // The file name without the .java extention
                fileChannel.close();
            } catch (FileNotFoundException e) {
                error(e);
            } catch (IOException e) {
                error(e);
            }
        }
    }//GEN-LAST:event_jButtonLoadActionPerformed
    
    private void jButtonCompileActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonCompileActionPerformed
        if (jTextFieldCustomBotName.getText().length() == 0) {
            notice("Please write a name for your bot");
        } else {
            saveCustomBot();
            
            notice("Ja i fr mig fandeme ikke til at lave compileren alene ~:-@");
            
            // Find the file of the bot
            Class cls = this.getClass();
            ProtectionDomain pDomain = cls.getProtectionDomain();
            CodeSource cSource = pDomain.getCodeSource();
            URL thisDir = cSource.getLocation();
            File file = new File(thisDir.getPath() + File.separator + "Bots" + File.separator + jTextFieldCustomBotName.getText() + ".java");
            
            // Convert the directory to a URL
            URL botDir = null;
            try {
                botDir = file.toURL();
            } catch (MalformedURLException e) {
                error(e);
            }
            URL[] botDirs = new URL[]{botDir};
            
            // Call the compiler
            boolean noCompileErrors = false;
            
            //            File file = new File(thisDir.getPath() + File.separator + "Bots" + File.separator + jTextFieldCustomBotName.getText() + ".class");
            //            sun.tools.javac.Main javac = new sun.tools.javac.Main(...);
            
            // If compilation went wrong
            if (!noCompileErrors) {
                //notice("Sorry but that just doesn't work, you stupid bastard");
                // If compilation went right, update the game
            } else {
                // Clear and reload all the available bot classes
                dlmAvailableBots.clear();
                listBots();
                
                // Create a new class loader with the directory
                ClassLoader cl = new URLClassLoader(botDirs);
                
                // Replace all instances of the bot with the new one
                for (int i = 0; i < dlmBotsInGame.size(); i++) {
                    
                    // Try to create a new instance of the bot
                    Bot bot = null;
                    
                    try {
                        Class botClass = cl.loadClass(file.getName().substring(0, file.getName().length() - 6)); // Remove the .class extension
                        bot = (Bot) botClass.newInstance();
                    } catch (ClassCastException e) {
                        error(e);
                    } catch (ClassNotFoundException e) {
                        error(e);
                    } catch (InstantiationException e) {
                        error(e);
                    } catch (IllegalAccessException e) {
                        error(e);
                    }
                    
                    // Replace the real bot with a new instance
                    Arena.FakeBot fakeBot = (Arena.FakeBot) dlmBotsInGame.get(i);
                    if (fakeBot.real.equals(bot)) {
                        game.kill(fakeBot.real);
                        fakeBot.toRealBot(bot);
                    }
                }
            }
        }
    }//GEN-LAST:event_jButtonCompileActionPerformed
    
    private void jTabbedPaneSetupStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jTabbedPaneSetupStateChanged
        if (oldComponentFocus == null) // Only happens once after initializing
            oldComponentFocus = jPanelGame;
        
        if (jTabbedPaneSetup.getSelectedComponent() != null && game != null) { // If setup has just been initialized
            
            // If the game panel is selected and the game is stopped, hide all bot markers
            if (jTabbedPaneSetup.getSelectedComponent().equals(jPanelGame) && game.gameStatus == game.STOPPED) {
                if (!jListArenas.isSelectionEmpty()) {
                    ArrayList objects = ((Arena) jListArenas.getSelectedValue()).objects;
                    for (int i = 0; i < objects.size(); i++) {
                        if (((Arena.FakeObject) objects.get(i)).isBot()) {
                            Arena.FakeBot bot = (Arena.FakeBot) objects.get(i);
                            if (bot.isBotMarker()) {
                                bot.hide();
                            }
                        }
                    }
                }
                
                // If the bots or arena panel is selected
            } else if (jTabbedPaneSetup.getSelectedComponent().equals(jPanelBots) || jTabbedPaneSetup.getSelectedComponent().equals(jPanelArena)) {
                
                // If game is stopped and the old and focus has just been lost from the game panel, unhide all bot markers
                if (game.gameStatus == game.STOPPED) {
                    if (oldComponentFocus.equals(jPanelGame) && !jListArenas.isSelectionEmpty()) {
                        ArrayList objects = ((Arena) jListArenas.getSelectedValue()).objects;
                        for (int i = 0; i < objects.size(); i++) {
                            if (((Arena.FakeObject) objects.get(i)).isBot()) {
                                Arena.FakeBot bot = (Arena.FakeBot) objects.get(i);
                                if (bot.isBotMarker()) {
                                    bot.unhide();
                                }
                            }
                        }
                    }
                } else { // Then game is running
                    jTabbedPaneSetup.setSelectedComponent(oldComponentFocus); // Do not allow to visit arena or bots panel
                }
            }
            
            oldComponentFocus = jTabbedPaneSetup.getSelectedComponent();
        } // End if setup has just been initialized
    }//GEN-LAST:event_jTabbedPaneSetupStateChanged
    
    private void jComboBoxOnTeamActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBoxOnTeamActionPerformed
        if (jListBotsInGame.getSelectedIndex() != -1) {
            Arena.FakeBot bot = (Arena.FakeBot) jListBotsInGame.getSelectedValue();
            bot.real.onTeam = (String) jComboBoxOnTeam.getSelectedItem();
            if (bot.real.onTeam.equals("Blue"))
                bot.setColor(Color.blue);
            else if (bot.real.onTeam.equals("Green"))
                bot.setColor(new Color(0, 180, 0)); // Dark green
            updateAddButton();
        }
    }//GEN-LAST:event_jComboBoxOnTeamActionPerformed
    
    private void jListBotsInGameValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_jListBotsInGameValueChanged
        if (jListBotsInGame.getSelectedIndex() == -1) {
            jButtonRemoveBot.setEnabled(false);
            jComboBoxOnTeam.setEnabled(false);
        } else {
            jButtonRemoveBot.setEnabled(true);
            jComboBoxOnTeam.setEnabled(true);
            Arena.FakeBot bot = (Arena.FakeBot) jListBotsInGame.getSelectedValue();
            jComboBoxOnTeam.getModel().setSelectedItem(bot.real.onTeam);
        }
    }//GEN-LAST:event_jListBotsInGameValueChanged
    
    void updateAddButton() {
        // Check for a free possision
        boolean foundFreePosition = false;
        if (!jListArenas.isSelectionEmpty()) {
            ArrayList objects = ((Arena) jListArenas.getSelectedValue()).objects;
            for (int i = 0; i < objects.size(); i++) {
                Arena.FakeObject object = (Arena.FakeObject) objects.get(i);
                if (object.isBotMarker()) {
                    foundFreePosition = true;
                    break;
                }
            }
        }
        
        // If a free position was found
        if (foundFreePosition)
            jButtonAddBot.setEnabled(true);
        else
            jButtonAddBot.setEnabled(false);
    }
    
    private void jListAvailableBotsValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_jListAvailableBotsValueChanged
        if (jListAvailableBots.getSelectedIndex() == -1 || jListArenas.getSelectedIndex() == -1) {
            jButtonAddBot.setEnabled(false);
        } else {
            updateAddButton();
        }
    }//GEN-LAST:event_jListAvailableBotsValueChanged
    
    private void jCheckBoxTeamplayActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxTeamplayActionPerformed
        if (jCheckBoxTeamplay.isSelected()) {
            game.isTeamPlayOn = true;
            jLabelNoteTeamplay.setVisible(false);
        } else {
            game.isTeamPlayOn = false;
            jLabelNoteTeamplay.setVisible(true);
        }
        setMayStartGame();
    }//GEN-LAST:event_jCheckBoxTeamplayActionPerformed
    
    private void jCheckBoxFriendlyFireActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxFriendlyFireActionPerformed
        if (jCheckBoxFriendlyFire.isSelected())
            game.isDamageOnFriendlyFire = true;
        else {
            game.isDamageOnFriendlyFire = false;
            jCheckBoxTeamplay.setSelected(true);
        }
    }//GEN-LAST:event_jCheckBoxFriendlyFireActionPerformed
    
    private void jCheckBoxNiceRenderingActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxNiceRenderingActionPerformed
        game.usingNiceRendering = jCheckBoxNiceRendering.isSelected();
    }//GEN-LAST:event_jCheckBoxNiceRenderingActionPerformed
    
    private void jCheckBoxTimerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxTimerActionPerformed
        game.timerIsEnabled = jCheckBoxTimer.isSelected();
    }//GEN-LAST:event_jCheckBoxTimerActionPerformed
    
    private void jCheckBoxRestartRoundActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxRestartRoundActionPerformed
        game.willStartNewRoundAfterThis = jCheckBoxRestartRound.isSelected();
        if (jCheckBoxRestartRound.isSelected() && game.gameStatus != game.STOPPED)
            jButtonEndRound.setEnabled(true);
        else
            jButtonEndRound.setEnabled(false);
    }//GEN-LAST:event_jCheckBoxRestartRoundActionPerformed
    
    private void jSpinnerTimeoutStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerTimeoutStateChanged
        game.timerStart = ((Number) jSpinnerTimeout.getValue()).intValue();
        if (game.gameStatus == game.STOPPED)
            game.timer = game.timerStart;
    }//GEN-LAST:event_jSpinnerTimeoutStateChanged
    
    private void jSpinnerGameSpeedStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerGameSpeedStateChanged
        game.gameSpeed = ((Number) jSpinnerGameSpeed.getValue()).intValue();
    }//GEN-LAST:event_jSpinnerGameSpeedStateChanged
    
    private void jButtonEndRoundActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonEndRoundActionPerformed
        game.timer = 1;
    }//GEN-LAST:event_jButtonEndRoundActionPerformed
    
    private void jButtonSearchForLayoutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSearchForLayoutActionPerformed
        JFileChooser fc = new JFileChooser();
        fc.addChoosableFileFilter(new javax.swing.filechooser.FileFilter() {
            public boolean accept(File file) {
                return file.getName().endsWith(".arena") || file.isDirectory();
            }
            public String getDescription() {
                return "*.arena";
            }
        }
        );
        fc.setMultiSelectionEnabled(true);
        fc.showOpenDialog(this);
        File[] files = fc.getSelectedFiles();
        if (files.length > 0) {
            listArenasFromFiles(files);
        }
    }//GEN-LAST:event_jButtonSearchForLayoutActionPerformed
    
    private void jButtonSearchForBotsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSearchForBotsActionPerformed
        // Find the Bots directory
        Class cls = this.getClass();
        ProtectionDomain pDomain = cls.getProtectionDomain();
        CodeSource cSource = pDomain.getCodeSource();
        URL thisDir = cSource.getLocation();
        File botsDir = new File(thisDir.getPath() + File.separator + "Bots" + File.separator);
        
        JFileChooser fc = new JFileChooser();
        fc.setCurrentDirectory(botsDir);
        fc.addChoosableFileFilter(new javax.swing.filechooser.FileFilter() {
            public boolean accept(File file) {
                return file.getName().endsWith(".class") || file.isDirectory();
            }
            public String getDescription() {
                return "*.class";
            }
        }
        );
        fc.setMultiSelectionEnabled(true);
        fc.showOpenDialog(this);
        File[] files = fc.getSelectedFiles();
        if (files.length > 0) {
            URL fileDir = null;
            try {
                fileDir = (new File(files[0].getParent())).toURL();
            } catch (MalformedURLException e) {
                error(e);
            }
            URL[] fileDirs = new URL[]{fileDir};
            ClassLoader cl = new URLClassLoader(fileDirs);
            listBotsFromFiles(files, cl);
        }
    }//GEN-LAST:event_jButtonSearchForBotsActionPerformed
    
    private void jButtonExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonExitActionPerformed
        System.exit(0);
    }//GEN-LAST:event_jButtonExitActionPerformed
    
    private void jSpinnerObstacleHeightStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerObstacleHeightStateChanged
        Arena.FakeObstacle object = ((Arena.FakeObstacle) jListObjects.getSelectedValue());
        object.setHeight(((Number) jSpinnerObstacleHeight.getValue()).intValue());
    }//GEN-LAST:event_jSpinnerObstacleHeightStateChanged
    
    private void jSpinnerObstacleWidthStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerObstacleWidthStateChanged
        Arena.FakeObstacle object = ((Arena.FakeObstacle) jListObjects.getSelectedValue());
        object.setWidth(((Number) jSpinnerObstacleWidth.getValue()).intValue());
    }//GEN-LAST:event_jSpinnerObstacleWidthStateChanged
    
    private void jSpinnerYStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerYStateChanged
        Arena.FakeObject object = ((Arena.FakeObject) jListObjects.getSelectedValue());
        int y = ((Number) jSpinnerY.getValue()).intValue();
        object.setY(y);
        if (object.getY() != y) {
            jSpinnerY.setValue(new Integer(object.getY()));
        }
    }//GEN-LAST:event_jSpinnerYStateChanged
    
    private void jSpinnerXStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerXStateChanged
        Arena.FakeObject object = ((Arena.FakeObject) jListObjects.getSelectedValue());
        int x = ((Number) jSpinnerX.getValue()).intValue();
        object.setX(x);
        if (object.getX() != x) {
            jSpinnerX.setValue(new Integer(object.getX()));
        }
    }//GEN-LAST:event_jSpinnerXStateChanged
    
    private void jButtonRemoveBotPositionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRemoveBotPositionActionPerformed
        Arena.FakeObject object = ((Arena.FakeObject) jListObjects.getSelectedValue());
        object.remove();
        dlmObjects.removeElement(object);
        if (dlmObjects.getSize() > 0) {
            jListObjects.setSelectedIndex(0);
            jListObjects.ensureIndexIsVisible(0);
        }
        
        if (object.isBot()) {
            Arena.FakeBot bot = (Arena.FakeBot) object;
            
            // Try to remove the bot from the "bots in game" list. If it was present, remove it from the statistics table
            if (dlmBotsInGame.removeElement(bot)) {
                for (int i = 0; i < dtmTable.getRowCount(); i++) {
                    if (dtmTable.getValueAt(i, dtmTable.findColumn("Name")) == bot) {
                        dtmTable.removeRow(i);
                        break;
                    } else if (i == dtmTable.getRowCount() - 1)
                        notice("BUG in jButtonRemoveBotActionPerformed!");
                }
            }
        }
        
        Arena arena = (Arena) jListArenas.getSelectedValue();
        arena.checkCollision();
    }//GEN-LAST:event_jButtonRemoveBotPositionActionPerformed
    
    private void jButtonRemoveObstacleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRemoveObstacleActionPerformed
        Arena.FakeObject object = ((Arena.FakeObject) jListObjects.getSelectedValue());
        object.remove();
        dlmObjects.removeElement(object);
        if (dlmObjects.getSize() > 0) {
            jListObjects.setSelectedIndex(0);
            jListObjects.ensureIndexIsVisible(0);
        }
        
        Arena arena = (Arena) jListArenas.getSelectedValue();
        arena.checkCollision();
    }//GEN-LAST:event_jButtonRemoveObstacleActionPerformed
    
    private void jButtonAddBotPositionActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddBotPositionActionPerformed
        Arena.FakeBot bot = ((Arena) jListArenas.getSelectedValue()).makeBot(20, 20, 0); // Get the arena and create a new bot. Don't care about its possisions
        dlmObjects.addElement(bot);
        jListObjects.setSelectedIndex(dlmObjects.indexOf(bot));
        jListObjects.ensureIndexIsVisible(jListObjects.getSelectedIndex());
    }//GEN-LAST:event_jButtonAddBotPositionActionPerformed
    
    private void jButtonAddObstacleActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddObstacleActionPerformed
        Arena.FakeObstacle obstacle = ((Arena) jListArenas.getSelectedValue()).makeObstacle(20, 20, 10, 10); // Get the arena and create a new obstacle. Don't care about its possisions
        dlmObjects.addElement(obstacle);
        jListObjects.setSelectedIndex(dlmObjects.indexOf(obstacle));
        jListObjects.ensureIndexIsVisible(jListObjects.getSelectedIndex());
    }//GEN-LAST:event_jButtonAddObstacleActionPerformed
    
    private void jTextFieldNameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldNameActionPerformed
        jTextFieldNameUpdated(false);
    }//GEN-LAST:event_jTextFieldNameActionPerformed
    
    private void jButtonStopGameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonStopGameActionPerformed
        game.reset();
    }//GEN-LAST:event_jButtonStopGameActionPerformed
    
    private void jButtonNewLayoutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonNewLayoutActionPerformed
        Arena arena = new Arena();
        arena.setName("Default");
        
        // Check if name "Default" is already taken. If so construct a default name that isn't
        int i = 0;
        while (dlmArenas.contains(arena)) {
            ++i;
            if (i <= 1) {
                arena.setName(arena.getName() + i);
            } else if (i <= 10) {
                arena.setName(arena.getName().substring(0, arena.getName().length() - 1) + i);
            } else if (i <= 100) {
                arena.setName(arena.getName().substring(0, arena.getName().length() - 2) + i);
            }
        }
        
        // Add the arena to the list and place focus on the text field
        dlmArenas.addElement(arena);
        jListArenas.setSelectedIndex(dlmArenas.indexOf(arena));
        jListArenas.ensureIndexIsVisible(dlmArenas.indexOf(arena));
        jTextFieldName.grabFocus();
        jTextFieldName.selectAll();
    }//GEN-LAST:event_jButtonNewLayoutActionPerformed
    
    private void jSpinnerArenaHeightStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerArenaHeightStateChanged
        Arena arena = ((Arena) jListArenas.getSelectedValue());
        int height = ((Number) jSpinnerArenaHeight.getValue()).intValue();
        arena.setHeight(height);
        game.setHeight(height);
    }//GEN-LAST:event_jSpinnerArenaHeightStateChanged
    
    private void jSpinnerArenaWidthStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerArenaWidthStateChanged
        Arena arena = ((Arena) dlmArenas.get(jListArenas.getSelectedIndex()));
        int width = ((Number) jSpinnerArenaWidth.getValue()).intValue();
        arena.setWidth(width);
        game.setWidth(width);
    }//GEN-LAST:event_jSpinnerArenaWidthStateChanged
    
    private void jSpinnerAngleStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jSpinnerAngleStateChanged
        Arena.FakeBot object = ((Arena.FakeBot) jListObjects.getSelectedValue());
        object.setAngle(((Number) jSpinnerAngle.getValue()).intValue());
    }//GEN-LAST:event_jSpinnerAngleStateChanged
    
    private void jListObjectsValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_jListObjectsValueChanged
        if (jListObjects.getSelectedIndex() == -1) {
            jSpinnerObstacleWidth.setEnabled(false);
            jSpinnerObstacleHeight.setEnabled(false);
            jSpinnerX.setEnabled(false);
            jSpinnerY.setEnabled(false);
            jSpinnerAngle.setEnabled(false);
        } else {
            updateAddButton();
            Arena.FakeObject object = ((Arena.FakeObject) dlmObjects.get(jListObjects.getSelectedIndex()));
            if (object.isBot()) {
                jSpinnerObstacleWidth.setEnabled(false);
                jSpinnerObstacleHeight.setEnabled(false);
                jSpinnerX.setEnabled(true);
                jSpinnerY.setEnabled(true);
                jSpinnerAngle.setEnabled(true);
                jButtonRemoveBotPosition.setEnabled(true);
                jButtonRemoveObstacle.setEnabled(false);
                
                // Make sure x and y is in the interval of the spinner model
                if (object.getX() < 25 || object.getX() > 5000)
                    object.setX(25);
                if (object.getY() < 25 || object.getY() > 5000)
                    object.setY(25);
                
                modelX = new SpinnerNumberModel(object.getX(), 25, 5000, spinnerStep);
                modelY = new SpinnerNumberModel(object.getY(), 25, 5000, spinnerStep);
                jSpinnerX.setModel(modelX);
                jSpinnerY.setModel(modelY);
                
                jSpinnerAngle.setValue(new Integer(((Arena.FakeBot) object).getAngle()));
                
            } else { // Is obstacle
                jSpinnerObstacleWidth.setEnabled(true);
                jSpinnerObstacleHeight.setEnabled(true);
                jSpinnerX.setEnabled(true);
                jSpinnerY.setEnabled(true);
                jSpinnerAngle.setEnabled(false);
                jButtonRemoveBotPosition.setEnabled(false);
                jButtonRemoveObstacle.setEnabled(true);
                
                // Make sure x and y is in the interval of the spinner model
                if (object.getX() < 5 || object.getX() > 5000)
                    object.setX(5);
                if (object.getY() < 5 || object.getY() > 5000)
                    object.setY(5);
                
                modelX = new SpinnerNumberModel(object.getX(), 5, 5000, spinnerStep);
                modelY = new SpinnerNumberModel(object.getY(), 5, 5000, spinnerStep);
                jSpinnerX.setModel(modelX);
                jSpinnerY.setModel(modelY);
                
                jSpinnerObstacleWidth.setValue(new Integer(((Arena.FakeObstacle) object).getWidth()));
                jSpinnerObstacleHeight.setValue(new Integer(((Arena.FakeObstacle) object).getHeight()));
            }
        }
    }//GEN-LAST:event_jListObjectsValueChanged
    
    private void jListArenasValueChanged(javax.swing.event.ListSelectionEvent evt) {//GEN-FIRST:event_jListArenasValueChanged
        dlmObjects.clear();
        dlmBotsInGame.clear();

        // Clear the statistics table
        for (int i = 0; i < dtmTable.getRowCount(); i++) {
            dtmTable.removeRow(i);
        }       
        
        game.reset(); // Remove all elements from the current game setup
        
        updateAddButton();
        
        if (jListArenas.getSelectedIndex() == -1) {
            jTextFieldName.setEnabled(false);
            jSpinnerArenaWidth.setEnabled(false);
            jSpinnerArenaHeight.setEnabled(false);
            jButtonRemoveLayout.setEnabled(false);
            jButtonAddObstacle.setEnabled(false);
            jButtonAddBotPosition.setEnabled(false);
            
            game.setWidth(500);
            game.setHeight(500);
        } else {
            jTextFieldName.setEnabled(true);
            jSpinnerArenaWidth.setEnabled(true);
            jSpinnerArenaHeight.setEnabled(true);
            jButtonRemoveLayout.setEnabled(true);
            jButtonAddObstacle.setEnabled(true);
            jButtonAddBotPosition.setEnabled(true);
            
            Arena arena = (Arena) dlmArenas.get(jListArenas.getSelectedIndex());
            jTextFieldName.setText(arena.getName());
            jSpinnerArenaWidth.setValue(new Integer(arena.getWidth()));
            jSpinnerArenaHeight.setValue(new Integer(arena.getHeight()));
            
            for (int i = 0; i < arena.objects.size(); i++) {
                dlmObjects.addElement(arena.objects.get(i));
            }
            
            game.setWidth(arena.getWidth());
            game.setHeight(arena.getHeight());
            arena.removeAll();
            arena.spawnAll();
            
            setMayStartGame();
        }
        jListObjects.repaint();
    }//GEN-LAST:event_jListArenasValueChanged
    
    private void jButtonRemoveLayoutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRemoveLayoutActionPerformed
        Arena arena = (Arena) dlmArenas.get(jListArenas.getSelectedIndex());
        removedArenas.add(arena);
        dlmArenas.removeElement(arena);
        if (dlmArenas.getSize() > 0) {
            jListArenas.setSelectedIndex(0);
            jListArenas.ensureIndexIsVisible(0);
        }
        updateAddButton();
    }//GEN-LAST:event_jButtonRemoveLayoutActionPerformed
    
    private void jButtonSaveAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonSaveAllActionPerformed
        for (int i = 0; i < dlmArenas.getSize(); i++) {
            ((Arena) dlmArenas.get(i)).save();
        }
        
        for (int i = 0; i < removedArenas.size(); i++) {
            ((Arena) removedArenas.get(i)).delete();
            removedArenas.remove(i);
        }
    }//GEN-LAST:event_jButtonSaveAllActionPerformed
    
    private void jTextFieldNameKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jTextFieldNameKeyTyped
        if (jTextFieldName.getText().length() < 1) {
            jTextFieldName.setText("Default");
            jTextFieldName.selectAll();
        }
    }//GEN-LAST:event_jTextFieldNameKeyTyped
    
    private void jTextFieldNameUpdated(boolean focusLost) {
        Arena arena = (Arena) dlmArenas.get(jListArenas.getSelectedIndex());
        
        // Check if text field is differene from arena name
        if (jListArenas.getSelectedIndex() != -1 && !arena.getName().equals(jTextFieldName.getText())) {
            Arena fakeArena = new Arena();
            fakeArena.setName(jTextFieldName.getText());
            
            // Check if the name is in use
            if (dlmArenas.contains(fakeArena)) {
                jTextFieldName.grabFocus();
                jTextFieldName.selectAll();
            } else {
                ((Arena) dlmArenas.get(jListArenas.getSelectedIndex())).setName(jTextFieldName.getText());
                jListArenas.repaint();
                jListArenas.requestFocus();
            }
        } else if (!focusLost) {
            jListArenas.requestFocus();
        }
    }
    
    private void jTextFieldNameFocusLost(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_jTextFieldNameFocusLost
        jTextFieldNameUpdated(true);
    }//GEN-LAST:event_jTextFieldNameFocusLost
    
    private void jButtonRemoveBotActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonRemoveBotActionPerformed
        int index = jListBotsInGame.getSelectedIndex();
        if (index != -1) {
            Arena.FakeBot bot = (Arena.FakeBot) dlmBotsInGame.get(index);
            bot.toBotMarker();
            dlmBotsInGame.remove(index);
            if (dlmBotsInGame.size() > 0) {
                if (index > 0)
                    jListBotsInGame.setSelectedIndex(index -1);
                else
                    jListBotsInGame.setSelectedIndex(index);
            }
            
            // Remove the bot from the statistics table
            for (int i = 0; i < dtmTable.getRowCount(); i++) {
                if (dtmTable.getValueAt(i, dtmTable.findColumn("Name")) == bot) {
                    dtmTable.removeRow(i);
                    break;
                } else if (i == dtmTable.getRowCount() - 1)
                    notice("BUG! In Setup.jButtonRemoveBotActionPerformed(): Could not find the bot: " + bot);
            }
            
            updateAddButton();
            setMayStartGame();
        }
    }//GEN-LAST:event_jButtonRemoveBotActionPerformed
    
    private void jButtonAddBotActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonAddBotActionPerformed
        if (jListArenas.getSelectedIndex() != -1) { // If an arena is selected
            // Find a free possision
            Arena.FakeBot bot = null;
            ArrayList objects = ((Arena) jListArenas.getSelectedValue()).objects;
            for (int i = 0; i < objects.size(); i++) {
                Arena.FakeObject object = (Arena.FakeObject) objects.get(i);
                if (object.isBotMarker()) {
                    bot = (Arena.FakeBot) object;
                    break;
                }
            }
            
            if (bot != null) { // If a free position was found
                // Fill in an instance of the selected bot class
                int index = jListAvailableBots.getSelectedIndex();
                Class botClass = (Class) dlmAvailableBots.get(index);
                try {
                    bot.toRealBot((Bot) botClass.newInstance());
                    
                    bot.real.onTeam = (String) jComboBoxOnTeam.getModel().getElementAt(0);
                    if (bot.real.onTeam.equals("Blue"))
                        bot.setColor(Color.blue);
                    else if (bot.real.onTeam.equals("Green"))
                        bot.setColor(Color.green);
                    
                    dlmBotsInGame.addElement(bot);
                    jListBotsInGame.setSelectedIndex(dlmBotsInGame.size() - 1);
                    jListBotsInGame.ensureIndexIsVisible(dlmBotsInGame.size() - 1);
                    
                    // Add the bot to the statistics table
                    Object[] rowData = {bot, new Integer(0), new Integer(0), new Integer(0), new Double(0)}; // Last one is pain, calculated in doubles for precision
                    dtmTable.addRow(rowData);
                    
                    updateAddButton(); // Check if the game is full now, and if so disable the "add bot" button
                    setMayStartGame(); // Check if the game may be startet
                    
                } catch (InstantiationException e) {
                    error(e);
                } catch (IllegalAccessException e) {
                    error(e);
                }
            }
        }
    }//GEN-LAST:event_jButtonAddBotActionPerformed
    
    void setMayStartGame() {
        //System.out.println("bots: " + game.nrOfBotsLeft + " teams: " + game.teamsLeft.size());
        jButtonStartGame.setEnabled(false);                                // Game may not be startet
        if (((Arena) jListArenas.getSelectedValue()).botsStuck.size() == 0 // No bots are stuck
        && ((game.isTeamPlayOn && game.teamsLeft.size() > 1)               // Team play is on and there are several teams in the game
        || (!game.isTeamPlayOn && game.nrOfBotsLeft > 1)))                 // Or team play is off and there are several bots in the game
            jButtonStartGame.setEnabled(true);                             // Game may be startet
    }
    
    private void jButtonStartGameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonStartGameActionPerformed
        if (game.gameStatus == game.STOPPED) {
            startGame();
            game.startGame();
        } else if (game.gameStatus == game.RUNNING) {
            game.gameStatus = game.PAUSED;
            jButtonStartGame.setText("Pause");
        } else if (game.gameStatus == game.PAUSED) {
            game.gameStatus = game.RUNNING;
            jButtonStartGame.setText("Resume");
        }
    }//GEN-LAST:event_jButtonStartGameActionPerformed
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroupShape;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButtonAddBot;
    private javax.swing.JButton jButtonAddBotPosition;
    private javax.swing.JButton jButtonAddObstacle;
    private javax.swing.JButton jButtonCompile;
    private javax.swing.JButton jButtonEndRound;
    private javax.swing.JButton jButtonExit;
    private javax.swing.JButton jButtonLoad;
    private javax.swing.JButton jButtonNewLayout;
    private javax.swing.JButton jButtonRemoveBot;
    private javax.swing.JButton jButtonRemoveBotPosition;
    private javax.swing.JButton jButtonRemoveLayout;
    private javax.swing.JButton jButtonRemoveObstacle;
    private javax.swing.JButton jButtonSave;
    private javax.swing.JButton jButtonSaveAll;
    private javax.swing.JButton jButtonSearchForBots;
    private javax.swing.JButton jButtonSearchForLayout;
    private javax.swing.JButton jButtonStartGame;
    private javax.swing.JButton jButtonStopGame;
    private javax.swing.JCheckBox jCheckBoxFriendlyFire;
    private javax.swing.JCheckBox jCheckBoxNiceRendering;
    private javax.swing.JCheckBox jCheckBoxRestartRound;
    private javax.swing.JCheckBox jCheckBoxTeamplay;
    private javax.swing.JCheckBox jCheckBoxTimer;
    private javax.swing.JComboBox jComboBoxOnTeam;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabelAngle;
    private javax.swing.JLabel jLabelArena;
    private javax.swing.JLabel jLabelAvailableBots;
    private javax.swing.JLabel jLabelBots;
    private javax.swing.JLabel jLabelBotsInGame;
    private javax.swing.JLabel jLabelCustomBot;
    private javax.swing.JLabel jLabelGame;
    private javax.swing.JLabel jLabelGameSpeed;
    private javax.swing.JLabel jLabelHeight;
    private javax.swing.JLabel jLabelLayouts;
    private javax.swing.JLabel jLabelName;
    private javax.swing.JLabel jLabelNoteTeamplay;
    private javax.swing.JLabel jLabelObstacleHeight;
    private javax.swing.JLabel jLabelObstacleWidth;
    private javax.swing.JLabel jLabelObstacles;
    private javax.swing.JLabel jLabelOnTeam;
    private javax.swing.JLabel jLabelStatistics;
    private javax.swing.JLabel jLabelTimeout;
    private javax.swing.JLabel jLabelWelcome;
    private javax.swing.JLabel jLabelWidth;
    private javax.swing.JLabel jLabelX;
    private javax.swing.JLabel jLabelY;
    private javax.swing.JList jListArenas;
    private javax.swing.JList jListAvailableBots;
    private javax.swing.JList jListBotsInGame;
    private javax.swing.JList jListObjects;
    private javax.swing.JPanel jPanelArena;
    private javax.swing.JPanel jPanelBots;
    private javax.swing.JPanel jPanelCustomBot;
    private javax.swing.JPanel jPanelGame;
    private javax.swing.JPanel jPanelHelp;
    private javax.swing.JPanel jPanelStatistics;
    private javax.swing.JScrollPane jScrollPaneAvailableBots;
    private javax.swing.JScrollPane jScrollPaneBotsInGame;
    private javax.swing.JScrollPane jScrollPaneCustomBot;
    private javax.swing.JScrollPane jScrollPaneHelp;
    private javax.swing.JScrollPane jScrollPaneLayouts;
    private javax.swing.JScrollPane jScrollPaneObstacles;
    private javax.swing.JScrollPane jScrollPaneStatistics;
    private javax.swing.JSeparator jSeparatorArena;
    private javax.swing.JSeparator jSeparatorBots;
    private javax.swing.JSeparator jSeparatorCustomBot;
    private javax.swing.JSeparator jSeparatorGame;
    private javax.swing.JSeparator jSeparatorHelp;
    private javax.swing.JSeparator jSeparatorStatistics;
    private javax.swing.JSpinner jSpinnerAngle;
    private javax.swing.JSpinner jSpinnerArenaHeight;
    private javax.swing.JSpinner jSpinnerArenaWidth;
    private javax.swing.JSpinner jSpinnerGameSpeed;
    private javax.swing.JSpinner jSpinnerObstacleHeight;
    private javax.swing.JSpinner jSpinnerObstacleWidth;
    private javax.swing.JSpinner jSpinnerTimeout;
    private javax.swing.JSpinner jSpinnerX;
    private javax.swing.JSpinner jSpinnerY;
    private javax.swing.JTabbedPane jTabbedPaneSetup;
    private javax.swing.JTable jTableStatistics;
    private javax.swing.JTextArea jTextAreaCustomBot;
    private javax.swing.JTextArea jTextAreaHelp;
    private javax.swing.JTextField jTextFieldCustomBotName;
    private javax.swing.JTextField jTextFieldName;
    // End of variables declaration//GEN-END:variables
}
