/*
 * Game.java
 *
 * Created on 19. februar 2004, 17:59
 *
 * Some of the initialization code has been inspired (i.e. blatently stolen) from the Arcs example from Sun:
 * http://java.sun.com/products/java-media/2D/samples/suite/Arcs_Curves/Arcs.java
 */


package Game;

import Bots.*;
import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.*;


/** The class where the action is */
public final class Game extends JFrame implements Runnable {
    
    
    int gameSpeed = 40;
    static int aiSpeed = 180;
    static boolean usingNiceRendering = false;
    static boolean isTeamPlayOn = false;
    static boolean isDamageOnFriendlyFire = true;
    private static final Game GAME = new Game(); // May only be created once to prevent exploitation from bots
    Setup setup;
    static final int GAMEOVER = 0;
    static final int PAUSED = 1;
    static final int RUNNING = 2;
    static final int STOPPED = 3;
    int gameStatus = STOPPED;
    static int nrOfBotsLeft = 0;
    static TreeSet teamsLeft = new TreeSet();
    boolean willStartNewRoundAfterThis = false;
    boolean timerIsEnabled = true;
    int timerStart = 2000; // Time (in game ticks, or steps) until game ends.
    int timer = timerStart; // Ticks down
    private int height = 500;
    private int width = 500;
    static final ArrayList spamQueue = new ArrayList();
    static int spamTimerStart = 75;
    static int spamTimer = 0;
    static private ArrayList objects = new ArrayList(); // Graphical objects in the game
    private BufferedImage bimg;
    private static final int titleFrameHack = 27; // This is to account for strange JFrame behaviour - the point (0,0) is somewhere in the title frame. This way the canvas is moved down just below the title frame.
    
    
    /** A new instance of Game may only be created by itself since it has been designed as a Singleton */
    private Game() {
        setSize(width, height + titleFrameHack); // See description of titleFrameHack
        setBackground(new Color(230, 255, 230)); // Very light green
        setTitle("The return of the killer pacmen! Okey someone fix a name please...");
        setLocation(440, 0); // Move window to the right
        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setVisible(true);
        
        Thread thread = new Thread(this);
        thread.setPriority(Thread.MAX_PRIORITY); // MIN_PRIORITY is 1, NORMAL_PRIORITY is 5 and MAX_PRIORITY is 10
        thread.start();
    }
    
    
    /** Called from the Start class to create a new instance of Game */
    public static void newGame(boolean usingNiceRendering) {
        GAME.usingNiceRendering = usingNiceRendering; // Creates a new game if it isn't allready running. Sets the rendering quality as well.
    }
    
    
    /** Set the height of the arena */
    void setHeight(int height) {
        this.height = height;
        setSize(width, this.height + titleFrameHack);
    }
    
    
    /** Set the width of the arena */
    void setWidth(int width) {
        this.width = width;
        setSize(width, height + titleFrameHack);
    }
    
    
    /** Automatically called when stating a new thread */
    public void run() {
        setup = new Setup(GAME);
        while (true) {
            step();
            repaint(); // Automatically calls paint(...)
            try {
                Thread.currentThread().sleep(gameSpeed); // Raise to slow the game down. Will make the game run more fluently on crappy computers
            } catch (InterruptedException e) { break; }
        }
    }
    
    
    /** Creates a 2D object */
    private Graphics2D createGraphics2D() {
        Graphics2D g2d = null;
        if (bimg == null || bimg.getWidth() != width || bimg.getHeight() != height) {
            bimg = (BufferedImage) createImage(width, height);
        }
        g2d = bimg.createGraphics();
        g2d.setBackground(getBackground());
        g2d.clearRect(0, 0, width, height);
        
        if (usingNiceRendering)
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON); // Nicer images, but slower rendering
        
        return g2d;
    }
    
    
    /** Called automatically when nessesary to repaint the screen */
    public void paint(Graphics g) {
        Graphics2D g2d = createGraphics2D();
        g2d.setStroke(new BasicStroke(2.0f)); // How a figure should be drawn. This sets the width of the drawn line segments
        draw(g2d);
        g2d.dispose();
        g.drawImage(bimg, 0, titleFrameHack, this); // Draw image and notify the JFrame (this). The canvas is moved down to account for strange JFrame behaviour -- se hack discription
    }
    
    
    /** Draws every object as they are now. Does not change anything in the objects */
    private void draw(Graphics2D g2d) {
        // Draw each object in the game
        for (int i = 0; i < objects.size(); i++) {
            DefaultObject object = (DefaultObject) objects.get(i);
            object.draw(g2d);
        }
        
        // Spam the screen with a message
        if (spamQueue.size() > 0) { // A string should be printet
            g2d.setFont(new Font("Comic Sans MS", Font.PLAIN, 22));
            g2d.setColor(Color.RED);
            g2d.drawString((String) spamQueue.get(0), 10, 250);
        }
        
        // Draw the "time left" string in the top left of the window (always drawn on top)
        if (timerIsEnabled && gameStatus != GAMEOVER) {
            g2d.setFont(new Font("Comic Sans MS", Font.PLAIN, 22));
            g2d.setColor(Color.RED);
            g2d.drawString("" + timer, 12, 4 + titleFrameHack);
        }
    }
    
    
    /** Moves on to the next tick. Calculates each new step for all objects in the game */
    private void step() {
        // Calculate if a string should be shown
        if (spamQueue.size() > 0) {         // A string should be printed
            if (spamTimer == 0)             // If a string has just been added
                spamTimer = spamTimerStart; // Then reset the timer
            spamTimer--;                    // Make the timer tick down by one
            if (spamTimer == 0) {           // If the current string should stop being shown
                spamQueue.remove(0);        // Remove the string
            }
        }
        
        // If the game is running, calculate a step for each object in the game
        if (gameStatus == RUNNING) {
            for (int i = 0; i < objects.size(); i++) {
                DefaultObject object = (DefaultObject) objects.get(i);
                object.step();
            }
        }
        
        if (gameStatus == GAMEOVER) {
            if (willStartNewRoundAfterThis) {
                if (spamQueue.size() == 0) {
                    setup.startGame();
                    startGame();
                }
            } else
                gameStatus = STOPPED;
        }
        
        // Make the timer tick down by one even if GAMEOVER. This is to know when to reset the game
        if (gameStatus == RUNNING)
            timer += -1;
        
        // Check if the game should end
        if (gameStatus == RUNNING && (timer == 0 || (timer > 0 && ((!isTeamPlayOn && nrOfBotsLeft == 1) || (isTeamPlayOn && teamsLeft.size() == 1))))) {
            timer = 0;
            
            DefaultObject winner = null;
            
            // Add a win to the statistics for all remaining bots
            for (int i = 0; i < objects.size(); i++) {
                DefaultObject testWinner = ((DefaultObject) objects.get(i));
                if (testWinner.isBot()) {
                    ((Bot) testWinner).statsAddWin();
                    winner = testWinner;
                }
            }
            
            if (!isTeamPlayOn && nrOfBotsLeft == 1) { // If team play is off and only one bot remains
                spam("The winner is: " + winner);
            } else if (isTeamPlayOn && teamsLeft.size() == 1) { // One team remains
                spam("The winning team is: " + teamsLeft.first());
            } else { // Time out
                spam("It is a draw!");
            }
            
            if (willStartNewRoundAfterThis)
                spam("New game begins"); // Put this in the spam queue
            
            if (!willStartNewRoundAfterThis && timer > - 2 * spamTimerStart) // If one messege was spamed = the game has ended and will not restart
                reset();
            if (timer > - 3 * spamTimerStart) { // If two messeges was spamed = if the game should restart itself
                reset();
                gameStatus = GAMEOVER;
            }
        }
    }
    
    
    /** Destroys everything */
    void reset() {
        for (int i = 0; i < objects.size(); i++) {
            ((DefaultObject) objects.get(i)).stopThread();
        }
        objects.clear();
        timer = timerStart;
        nrOfBotsLeft = 0;
        teamsLeft.clear();
        gameStatus = STOPPED;
        if (setup != null) // Only false when initializing because the lists change. Should not have any impact
            setup.gameEnded();
    }
    
    
    /** Show a big message on the game screen */
    static void spam(String message) {
        spamQueue.add(message);
    }
    
    
    /** Start the event-listener of all objects in the game */
    void startGame() {
        for (int i = 0; i < objects.size(); i++) {
            ((DefaultObject) objects.get(i)).throwEventOnStart();
        }
        gameStatus = RUNNING;
    }
    
    
    /** Places an object on the game world according to its coordinates */
    static void spawn(DefaultObject object) {
        object.setGame(GAME);
        if (object.isBot() && object.isCustom()) { // It is not a marker
            nrOfBotsLeft++;
            teamsLeft.add(object.onTeam); // Insures there are no duplicates
        }
        objects.add(object);
    }
    
    
    /** Removes an object from the game world */
    static void kill(DefaultObject object) {
        // Kill the object
        object.stopThread();
        objects.remove(object);
        
        if (object.isBot() && object.isCustom()) {
            nrOfBotsLeft--;
            // Check if this is the last bot on its team
            boolean lastOnTeam = true;
            
            for (Iterator i = objects.iterator(); i.hasNext(); ) {
                DefaultObject bot = (DefaultObject) i.next();
                if (bot.isBot() && bot.isCustom() && bot.onTeam == object.onTeam)
                    lastOnTeam = false;
            }
            
            // If it is the last on team, remove the team
            if (lastOnTeam = true)
                teamsLeft.remove(object.onTeam);
        }
        
    }
    
    
    /** Returns the objects in the game window */
    static ArrayList getObjects() {
        return objects;
    }
    
}